# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the ios_snmp_server module
"""


class Snmp_serverArgs(object):  # pylint: disable=R0903
    """The arg spec for the ios_snmp_server module"""

    argument_spec = {
        "config": {
            "options": {
                "accounting": {
                    "options": {"command": {"type": "str"}},
                    "type": "dict",
                },
                "cache": {"type": "int"},
                "chassis_id": {"type": "str"},
                "communities": {
                    "elements": "dict",
                    "options": {
                        "acl_v4": {"type": "str"},
                        "acl_v6": {"type": "str"},
                        "name": {"type": "str"},
                        "ro": {"type": "bool"},
                        "rw": {"type": "bool"},
                        "view": {"type": "str"},
                    },
                    "type": "list",
                },
                "contact": {"type": "str"},
                "context": {"elements": "str", "type": "list"},
                "drop": {
                    "options": {
                        "unknown_user": {"type": "bool"},
                        "vrf_traffic": {"type": "bool"},
                    },
                    "type": "dict",
                },
                "engine_id": {
                    "elements": "dict",
                    "options": {
                        "id": {"type": "str"},
                        "local": {"type": "bool"},
                        "remote": {
                            "options": {
                                "host": {"type": "str"},
                                "udp_port": {"type": "int"},
                                "vrf": {"type": "str"},
                            },
                            "type": "dict",
                        },
                    },
                    "type": "list",
                },
                "file_transfer": {
                    "options": {
                        "access_group": {"type": "str"},
                        "protocol": {"type": "list", "elements": "str"},
                    },
                    "type": "dict",
                },
                "groups": {
                    "elements": "dict",
                    "options": {
                        "context": {"type": "str"},
                        "match": {"choices": ["exact", "prefix"], "type": "str"},
                        "version_option": {
                            "choices": ["auth", "noauth", "priv"],
                            "type": "str",
                        },
                        "group": {"type": "str"},
                        "notify": {"type": "str"},
                        "read": {"type": "str"},
                        "version": {"choices": ["v1", "v3", "v2c"], "type": "str"},
                        "write": {"type": "str"},
                        "acl_v4": {"type": "str"},
                        "acl_v6": {"type": "str"},
                    },
                    "type": "list",
                },
                "hosts": {
                    "elements": "dict",
                    "options": {
                        "host": {"type": "str"},
                        "informs": {"type": "bool"},
                        "community_string": {"type": "str"},
                        "traps": {"type": "list", "elements": "str"},
                        "version": {"choices": ["1", "2c", "3"], "type": "str"},
                        "version_option": {
                            "choices": ["auth", "noauth", "priv"],
                            "type": "str",
                        },
                        "vrf": {"type": "str"},
                    },
                    "type": "list",
                },
                "if_index": {"type": "bool"},
                "inform": {
                    "options": {
                        "pending": {"type": "int"},
                        "retries": {"type": "int"},
                        "timeout": {"type": "int"},
                    },
                    "type": "dict",
                },
                "ip": {
                    "options": {
                        "dscp": {"type": "int"},
                        "precedence": {"type": "int"},
                    },
                    "type": "dict",
                },
                "location": {"type": "str"},
                "manager": {"type": "int"},
                "packet_size": {"type": "int"},
                "password_policy": {
                    "elements": "dict",
                    "no_log": False,
                    "options": {
                        "change": {"type": "int"},
                        "digits": {"type": "int"},
                        "lower_case": {"type": "int"},
                        "max_len": {"type": "int"},
                        "min_len": {"type": "int"},
                        "policy_name": {"type": "str"},
                        "special_char": {"type": "int"},
                        "upper_case": {"type": "int"},
                        "username": {"type": "str"},
                    },
                    "type": "list",
                },
                "queue_length": {"type": "int"},
                "source_interface": {"type": "str"},
                "system_shutdown": {"type": "bool"},
                "trap_source": {"type": "str"},
                "trap_timeout": {"type": "int"},
                "traps": {
                    "options": {
                        "aaa_server": {"type": "bool"},
                        "auth_framework": {
                            "options": {
                                "sec_violation": {"type": "bool"},
                                "enable": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "bfd": {
                            "options": {
                                "enable": {"type": "bool"},
                                "session_down": {"type": "bool"},
                                "session_up": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "bgp": {
                            "options": {
                                "cbgp2": {"type": "bool"},
                                "enable": {"type": "bool"},
                                "state_changes": {
                                    "options": {
                                        "all": {"type": "bool"},
                                        "backward_trans": {"type": "bool"},
                                        "limited": {"type": "bool"},
                                        "enable": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "threshold": {
                                    "options": {"prefix": {"type": "bool"}},
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "bridge": {
                            "options": {
                                "newroot": {"type": "bool"},
                                "enable": {"type": "bool"},
                                "topologychange": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "bulkstat": {
                            "options": {
                                "enable": {"type": "bool"},
                                "collection": {"type": "bool"},
                                "transfer": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "call_home": {
                            "options": {
                                "enable": {"type": "bool"},
                                "message_send_fail": {"type": "bool"},
                                "server_fail": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "casa": {"type": "bool"},
                        "cef": {
                            "options": {
                                "inconsistency": {"type": "bool"},
                                "peer_fib_state_change": {"type": "bool"},
                                "peer_state_change": {"type": "bool"},
                                "resource_failure": {"type": "bool"},
                                "enable": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "cnpd": {"type": "bool"},
                        "config": {"type": "bool"},
                        "config_copy": {"type": "bool"},
                        "config_ctid": {"type": "bool"},
                        "cpu": {
                            "options": {
                                "enable": {"type": "bool"},
                                "threshold": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "dhcp": {"type": "bool"},
                        "dlsw": {
                            "options": {
                                "circuit": {"type": "bool"},
                                "enable": {"type": "bool"},
                                "tconn": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "eigrp": {"type": "bool"},
                        "entity": {"type": "bool"},
                        "entity_diag": {
                            "options": {
                                "boot_up_fail": {"type": "bool"},
                                "enable": {"type": "bool"},
                                "hm_test_recover": {"type": "bool"},
                                "hm_thresh_reached": {"type": "bool"},
                                "scheduled_test_fail": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "entity_perf": {
                            "options": {
                                "enable": {"type": "bool"},
                                "throughput_notif": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "entity_state": {"type": "bool"},
                        "energywise": {"type": "bool"},
                        "envmon": {
                            "options": {
                                "enable": {"type": "bool"},
                                "fan": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "shutdown": {"type": "bool"},
                                        "status": {"type": "bool"},
                                        "supply": {"type": "bool"},
                                        "temperature": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "fan_enable": {"type": "bool"},
                                "shutdown": {"type": "bool"},
                                "status": {"type": "bool"},
                                "supply": {"type": "bool"},
                                "temperature": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "errdisable": {"type": "bool"},
                        "ethernet": {
                            "options": {
                                "cfm": {
                                    "options": {
                                        "alarm": {"type": "bool"},
                                        "cc": {
                                            "type": "dict",
                                            "options": {
                                                "config": {"type": "bool"},
                                                "cross_connect": {"type": "bool"},
                                                "loop": {"type": "bool"},
                                                "mep_down": {"type": "bool"},
                                                "mep_up": {"type": "bool"},
                                            },
                                        },
                                        "crosscheck": {
                                            "type": "dict",
                                            "options": {
                                                "mep_missing": {"type": "bool"},
                                                "mep_unknown": {"type": "bool"},
                                                "service_up": {"type": "bool"},
                                            },
                                        },
                                    },
                                    "type": "dict",
                                },
                                "evc": {
                                    "options": {
                                        "create": {"type": "bool"},
                                        "delete": {"type": "bool"},
                                        "status": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "ether_oam": {"type": "bool"},
                        "event_manager": {"type": "bool"},
                        "flash": {
                            "options": {
                                "enable": {"type": "bool"},
                                "insertion": {"type": "bool"},
                                "lowspace": {"type": "bool"},
                                "removal": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "flex_links": {
                            "options": {
                                "enable": {"type": "bool"},
                                "status": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "firewall": {
                            "options": {
                                "enable": {"type": "bool"},
                                "serverstatus": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "flowmon": {"type": "bool"},
                        "frame_relay": {
                            "options": {
                                "enable": {"type": "bool"},
                                "subif": {
                                    "options": {
                                        "count": {"type": "int"},
                                        "interval": {"type": "int"},
                                        "enable": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "fru_ctrl": {"type": "bool"},
                        "hsrp": {"type": "bool"},
                        "ike": {
                            "options": {
                                "policy": {
                                    "options": {
                                        "add": {"type": "bool"},
                                        "delete": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "tunnel": {
                                    "options": {
                                        "start": {"type": "bool"},
                                        "stop": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "ipmulticast": {"type": "bool"},
                        "ipsec": {
                            "options": {
                                "cryptomap": {
                                    "options": {
                                        "add": {"type": "bool"},
                                        "attach": {"type": "bool"},
                                        "delete": {"type": "bool"},
                                        "detach": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "too_many_sas": {"type": "bool"},
                                "tunnel": {
                                    "options": {
                                        "start": {"type": "bool"},
                                        "stop": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "ipsla": {"type": "bool"},
                        "isis": {"type": "bool"},
                        "l2tc": {
                            "options": {
                                "enable": {"type": "bool"},
                                "sys_threshold": {"type": "bool"},
                                "threshold": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "l2tun": {
                            "options": {
                                "pseudowire_status": {"type": "bool"},
                                "session": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "license": {"type": "bool"},
                        "lisp": {"type": "bool"},
                        "local_auth": {"type": "bool"},
                        "mac_notification": {
                            "options": {
                                "enable": {"type": "bool"},
                                "change": {"type": "bool"},
                                "move": {"type": "bool"},
                                "threshold": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "memory": {
                            "options": {
                                "enable": {"type": "bool"},
                                "bufferpeak": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "mpls": {
                            "options": {
                                "fast_reroute": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "protected": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "ldp": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "pv_limit": {"type": "bool"},
                                        "session_down": {"type": "bool"},
                                        "session_up": {"type": "bool"},
                                        "threshold": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "rfc": {
                                    "options": {
                                        "ldp": {
                                            "options": {
                                                "enable": {"type": "bool"},
                                                "pv_limit": {"type": "bool"},
                                                "session_down": {"type": "bool"},
                                                "session_up": {"type": "bool"},
                                                "threshold": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "traffic_eng": {
                                            "options": {
                                                "enable": {"type": "bool"},
                                                "down": {"type": "bool"},
                                                "reoptimized": {"type": "bool"},
                                                "reroute": {"type": "bool"},
                                                "up": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                        "vpn": {
                                            "options": {
                                                "enable": {"type": "bool"},
                                                "illegal_label": {"type": "bool"},
                                                "max_thresh_cleared": {
                                                    "type": "bool",
                                                },
                                                "max_threshold": {"type": "bool"},
                                                "mid_threshold": {"type": "bool"},
                                                "vrf_down": {"type": "bool"},
                                                "vrf_up": {"type": "bool"},
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                                "traffic_eng": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "down": {"type": "bool"},
                                        "reroute": {"type": "bool"},
                                        "up": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "vpn": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "illegal_label": {"type": "bool"},
                                        "max_thresh_cleared": {"type": "bool"},
                                        "max_threshold": {"type": "bool"},
                                        "mid_threshold": {"type": "bool"},
                                        "vrf_down": {"type": "bool"},
                                        "vrf_up": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "mpls_vpn": {"type": "bool"},
                        "msdp": {"type": "bool"},
                        "mvpn": {"type": "bool"},
                        "nhrp": {
                            "options": {
                                "enable": {"type": "bool"},
                                "nhc": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "down": {"type": "bool"},
                                        "up": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "nhp": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "down": {"type": "bool"},
                                        "up": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "nhs": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "down": {"type": "bool"},
                                        "up": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "quota_exceeded": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "ospf": {
                            "options": {
                                "cisco_specific": {
                                    "options": {
                                        "error": {"type": "bool"},
                                        "lsa": {"type": "bool"},
                                        "retransmit": {"type": "bool"},
                                        "state_change": {
                                            "options": {
                                                "nssa_trans_change": {
                                                    "type": "bool",
                                                },
                                                "shamlink": {
                                                    "options": {
                                                        "interface": {
                                                            "type": "bool",
                                                        },
                                                        "neighbor": {
                                                            "type": "bool",
                                                        },
                                                    },
                                                    "type": "dict",
                                                },
                                            },
                                            "type": "dict",
                                        },
                                    },
                                    "type": "dict",
                                },
                                "error": {"type": "bool"},
                                "retransmit": {"type": "bool"},
                                "lsa": {"type": "bool"},
                                "state_change": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "ospfv3": {
                            "options": {
                                "errors": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "bad_packet": {"type": "bool"},
                                        "config_error": {"type": "bool"},
                                        "virt_bad_packet": {"type": "bool"},
                                        "virt_config_error": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                                "rate_limit": {"type": "int"},
                                "state_change": {
                                    "options": {
                                        "enable": {"type": "bool"},
                                        "if_state_change": {"type": "bool"},
                                        "neighbor_restart_helper_status_change": {
                                            "type": "bool",
                                        },
                                        "neighbor_state_change": {"type": "bool"},
                                        "nssa_translator_status_change": {
                                            "type": "bool",
                                        },
                                        "restart_status_change": {"type": "bool"},
                                        "virtif_state_change": {"type": "bool"},
                                        "vn_restart_helper_status_change": {
                                            "type": "bool",
                                        },
                                        "vn_state_change": {"type": "bool"},
                                    },
                                    "type": "dict",
                                },
                            },
                            "type": "dict",
                        },
                        "pim": {
                            "options": {
                                "invalid_pim_message": {"type": "bool"},
                                "neighbor_change": {"type": "bool"},
                                "rp_mapping_change": {"type": "bool"},
                                "enable": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "pki": {"type": "bool"},
                        "port_security": {"type": "bool"},
                        "power_ethernet": {
                            "options": {
                                "enable": {"type": "bool"},
                                "group": {
                                    "options": {
                                        "slot_id": {"type": "int"},
                                        "threshold": {"type": "int"},
                                    },
                                    "elements": "dict",
                                    "type": "list",
                                },
                                "police": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "pw_vc": {"type": "bool"},
                        "rep": {"type": "bool"},
                        "rsvp": {"type": "bool"},
                        "rf": {"type": "bool"},
                        "smart_license": {
                            "options": {
                                "enable": {"type": "bool"},
                                "entitlement": {"type": "bool"},
                                "global": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "snmp": {
                            "options": {
                                "authentication": {"type": "bool"},
                                "coldstart": {"type": "bool"},
                                "linkdown": {"type": "bool"},
                                "linkup": {"type": "bool"},
                                "warmstart": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "stackwise": {"type": "bool"},
                        "stpx": {
                            "options": {
                                "enable": {"type": "bool"},
                                "inconsistency": {"type": "bool"},
                                "loop_inconsistency": {"type": "bool"},
                                "root_inconsistency": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "syslog": {"type": "bool"},
                        "transceiver_all": {"type": "bool"},
                        "trustsec": {
                            "options": {
                                "authz_file_error": {"type": "bool"},
                                "cache_file_error": {"type": "bool"},
                                "enable": {"type": "bool"},
                                "keystore_file_error": {"type": "bool"},
                                "keystore_sync_fail": {"type": "bool"},
                                "random_number_fail": {"type": "bool"},
                                "src_entropy_fail": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "trustsec_interface": {
                            "options": {
                                "enable": {"type": "bool"},
                                "authc_fail": {"type": "bool"},
                                "authz_fail": {"type": "bool"},
                                "sap_fail": {"type": "bool"},
                                "supplicant_fail": {"type": "bool"},
                                "unauthorized": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "trustsec_policy": {
                            "options": {
                                "enable": {"type": "bool"},
                                "authz_sgacl_fail": {"type": "bool"},
                                "peer_policy_updated": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "trustsec_server": {
                            "options": {
                                "enable": {"type": "bool"},
                                "provision_secret": {"type": "bool"},
                                "radius_server": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "trustsec_sxp": {
                            "options": {
                                "enable": {"type": "bool"},
                                "binding_conflict": {"type": "bool"},
                                "binding_err": {"type": "bool"},
                                "binding_expn_fail": {"type": "bool"},
                                "conn_config_err": {"type": "bool"},
                                "conn_down": {"type": "bool"},
                                "conn_srcaddr_err": {"type": "bool"},
                                "conn_up": {"type": "bool"},
                                "msg_parse_err": {"type": "bool"},
                                "oper_nodeid_change": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "tty": {"type": "bool"},
                        "udld": {
                            "options": {
                                "enable": {"type": "bool"},
                                "link_fail_rpt": {"type": "bool"},
                                "status_change": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "vlan_membership": {"type": "bool"},
                        "vlancreate": {"type": "bool"},
                        "vlandelete": {"type": "bool"},
                        "vrfmib": {
                            "options": {
                                "vrf_up": {"type": "bool"},
                                "vrf_down": {"type": "bool"},
                                "vnet_trunk_up": {"type": "bool"},
                                "vnet_trunk_down": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "vrrp": {"type": "bool"},
                        "vswitch": {
                            "options": {
                                "dual_active": {"type": "bool"},
                                "enable": {"type": "bool"},
                                "vsl": {"type": "bool"},
                            },
                            "type": "dict",
                        },
                        "vtp": {"type": "bool"},
                    },
                    "type": "dict",
                },
                "users": {
                    "elements": "dict",
                    "options": {
                        "acl_v6": {"type": "str"},
                        "acl_v4": {"type": "str"},
                        "authentication": {
                            "no_log": False,
                            "type": "dict",
                            "options": {
                                "algorithm": {
                                    "type": "str",
                                    "choices": ["md5", "sha", "sha-2"],
                                },
                                "auth_option": {"type": "int"},
                                "password": {
                                    "type": "str",
                                    "no_log": True,
                                },
                            },
                        },
                        "encryption": {
                            "no_log": False,
                            "type": "dict",
                            "options": {
                                "priv": {
                                    "type": "str",
                                    "choices": ["3des", "aes", "des"],
                                },
                                "priv_option": {"type": "str"},
                                "password": {
                                    "type": "str",
                                    "no_log": True,
                                },
                            },
                        },
                        "group": {"type": "str"},
                        "remote": {"type": "str"},
                        "udp_port": {"type": "int"},
                        "username": {"type": "str"},
                        "version": {"choices": ["v1", "v2c", "v3"], "type": "str"},
                        "version_option": {
                            "choices": ["encrypted"],
                            "type": "str",
                        },
                        "vrf": {"type": "str"},
                    },
                    "type": "list",
                },
                "views": {
                    "elements": "dict",
                    "options": {
                        "excluded": {"type": "bool"},
                        "family_name": {"type": "str"},
                        "included": {"type": "bool"},
                        "name": {"type": "str"},
                    },
                    "type": "list",
                },
            },
            "type": "dict",
        },
        "running_config": {"type": "str"},
        "state": {
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "parsed",
                "gathered",
                "rendered",
            ],
            "default": "merged",
            "type": "str",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
#
# Copyright (c) 2016 Matt Davis, <mdavis@ansible.com>
#                    Chris Houseknecht, <house@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_resourcegroup_info

version_added: "0.1.2"

short_description: Get resource group facts

description:
    - Get facts for a specific resource group or all resource groups.

options:
    name:
        description:
            - Limit results to a specific resource group.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str
    list_resources:
        description:
            - List all resources under the resource group.
            - Note this will cost network overhead for each resource group. Suggest use this when I(name) set.
        type: bool


extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Chris Houseknecht (@chouseknecht)
    - Matt Davis (@nitzmahone)

'''

EXAMPLES = '''
- name: Get facts for one resource group
  azure_rm_resourcegroup_info:
    name: myResourceGroup

- name: Get facts for all resource groups
  azure_rm_resourcegroup_info:

- name: Get facts by tags
  azure_rm_resourcegroup_info:
    tags:
      - testing
      - foo:bar

- name: Get facts for one resource group including resources it contains
  azure_rm_resourcegroup_info:
    name: myResourceGroup
    list_resources: true
'''
RETURN = '''
resourcegroups:
    description:
        - List of resource group dicts.
    returned: always
    type: list
    contains:
        id:
            description:
                - Resource id.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroup/myResourceGroup"
        name:
            description:
                - Resource group name.
            returned: always
            type: str
            sample: foo
        tags:
            description:
                - Tags assigned to resource group.
            returned: always
            type: dict
            sample: { "tag": "value" }
        resources:
            description:
                - List of resources under the resource group.
            returned: when I(list_resources=yes).
            type: list
            contains:
                id:
                    description:
                        - Resource id.
                    returned: always
                    type: str
                    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/virtualMa
                             chines/myVirtualMachine"
                name:
                    description:
                        - Resource name.
                    returned: always
                    type: str
                    sample: myVirtualMachine
                location:
                    description:
                        - Resource region.
                    returned: always
                    type: str
                    sample: eastus
                type:
                    description:
                        - Resource type.
                    returned: always
                    type: str
                    sample: "Microsoft.Compute/virtualMachines"
                tags:
                    description:
                        - Tags to assign to the managed disk.
                    returned: always
                    type: dict
                    sample: { "tag": "value" }
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


AZURE_OBJECT_CLASS = 'ResourceGroup'


class AzureRMResourceGroupInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            tags=dict(type='list', elements='str'),
            list_resources=dict(type='bool')
        )

        self.results = dict(
            changed=False,
            resourcegroups=[]
        )

        self.name = None
        self.tags = None
        self.list_resources = None

        super(AzureRMResourceGroupInfo, self).__init__(self.module_arg_spec,
                                                       supports_check_mode=True,
                                                       supports_tags=False,
                                                       facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            result = self.get_item()
        else:
            result = self.list_items()

        if self.list_resources:
            for item in result:
                item['resources'] = self.list_by_rg(item['name'])

        self.results['resourcegroups'] = result

        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.name))
        item = None
        result = []

        try:
            item = self.rm_client.resource_groups.get(self.name)
        except ResourceNotFoundError:
            pass

        if item and self.has_tags(item.tags, self.tags):
            result = [self.serialize_obj(item, AZURE_OBJECT_CLASS)]

        return result

    def list_items(self):
        self.log('List all items')
        try:
            response = self.rm_client.resource_groups.list()
        except Exception as exc:
            self.fail("Failed to list all items - {0}".format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(self.serialize_obj(item, AZURE_OBJECT_CLASS))
        return results

    def list_by_rg(self, name):
        self.log('List resources under resource group')
        results = []
        try:
            response = self.rm_client.resources.list_by_resource_group(name)
            while True:
                results.append(response.next().as_dict())
        except StopIteration:
            pass
        except Exception as exc:
            self.fail('Error when listing resources under resource group {0}: {1}'.format(name, exc.message or str(exc)))
        return results


def main():
    AzureRMResourceGroupInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_cellular_gateway_esims_service_providers_accounts_communication_plans_info
short_description: Information module for organizations _cellular _gateway _esims
  _service _providers _accounts _communication _plans
description:
  - Get all organizations _cellular _gateway _esims _service _providers _accounts
    _communication _plans.
  - The communication plans available for a given provider.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  accountIds:
    description:
      - AccountIds query parameter. Account IDs that communication plans will be fetched
        for.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for cellularGateway getOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlans
    description: Complete reference of the getOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlans
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-cellular-gateway-esims-service-providers-accounts-communication-plans
notes:
  - SDK Method used are
    cellular_gateway.CellularGateway.get_organization_cellular_gateway_esims_service_providers_accounts_communication_plans,
  - Paths used are
    get /organizations/{organizationId}/cellularGateway/esims/serviceProviders/accounts/communicationPlans,
"""

EXAMPLES = r"""
- name: Get all organizations _cellular _gateway _esims _service _providers _accounts
    _communication _plans
  cisco.meraki.organizations_cellular_gateway_esims_service_providers_accounts_communication_plans_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    accountIds: []
    organizationId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "accountId": "string",
          "apns": [
            {
              "name": "string"
            }
          ],
          "name": "string"
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
#
# Copyright (C) 2019 Huawei
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

###############################################################################
# Documentation
###############################################################################

DOCUMENTATION = r"""
module: hwc_vpc_route
description:
  - VPC route management.
short_description: Creates a resource of VPC/Route in Huawei Cloud
notes:
  - If O(id) option is provided, it takes precedence over O(destination), O(vpc_id), O(type), and O(next_hop) for route selection.
  - O(destination), O(vpc_id), O(type) and O(next_hop) are used for route selection. If more than one route with this options
    exists, execution is aborted.
  - No parameter support updating. If one of option is changed, the module creates a new resource.
version_added: '0.2.0'
author: Huawei Inc. (@huaweicloud)
requirements:
  - keystoneauth1 >= 3.6.0
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  state:
    description:
      - Whether the given object should exist in Huawei Cloud.
    type: str
    choices: ['present', 'absent']
    default: 'present'
  destination:
    description:
      - Specifies the destination IP address or CIDR block.
    type: str
    required: true
  next_hop:
    description:
      - Specifies the next hop. The value is VPC peering connection ID.
    type: str
    required: true
  vpc_id:
    description:
      - Specifies the VPC ID to which route is added.
    type: str
    required: true
  type:
    description:
      - Specifies the type of route.
    type: str
    default: 'peering'
extends_documentation_fragment:
  - community.general._hwc
  - community.general._attributes
"""

EXAMPLES = r"""
# create a peering connect
- name: Create a local vpc
  hwc_network_vpc:
    cidr: "192.168.0.0/16"
    name: "ansible_network_vpc_test_local"
  register: vpc1
- name: Create a peering vpc
  hwc_network_vpc:
    cidr: "192.168.0.0/16"
    name: "ansible_network_vpc_test_peering"
  register: vpc2
- name: Create a peering connect
  hwc_vpc_peering_connect:
    local_vpc_id: "{{ vpc1.id }}"
    name: "ansible_network_peering_test"
    filters:
      - "name"
    peering_vpc:
      vpc_id: "{{ vpc2.id }}"
  register: connect
- name: Create a route
  community.general.hwc_vpc_route:
    vpc_id: "{{ vpc1.id }}"
    destination: "192.168.0.0/16"
    next_hop: "{{ connect.id }}"
"""

RETURN = r"""
id:
  description:
    - UUID of the route.
  type: str
  returned: success
destination:
  description:
    - Specifies the destination IP address or CIDR block.
  type: str
  returned: success
next_hop:
  description:
    - Specifies the next hop. The value is VPC peering connection ID.
  type: str
  returned: success
vpc_id:
  description:
    - Specifies the VPC ID to which route is added.
  type: str
  returned: success
type:
  description:
    - Specifies the type of route.
  type: str
  returned: success
"""

from ansible_collections.community.general.plugins.module_utils._hwc_utils import (
    Config,
    HwcClientException,
    HwcModule,
    are_different_dicts,
    build_path,
    get_region,
    is_empty_value,
    navigate_value,
)


def build_module():
    return HwcModule(
        argument_spec=dict(
            state=dict(default="present", choices=["present", "absent"], type="str"),
            destination=dict(type="str", required=True),
            next_hop=dict(type="str", required=True),
            vpc_id=dict(type="str", required=True),
            type=dict(type="str", default="peering"),
            id=dict(type="str"),
        ),
        supports_check_mode=True,
    )


def main():
    """Main function"""

    module = build_module()
    config = Config(module, "vpc")

    try:
        resource = None
        if module.params.get("id"):
            resource = get_resource_by_id(config)
            if module.params["state"] == "present":
                opts = user_input_parameters(module)
                if are_different_dicts(resource, opts):
                    raise Exception(
                        f"Cannot change option from ({resource}) to ({opts}) for an existing route.({config.module.params['id']})"
                    )
        else:
            v = search_resource(config)
            if len(v) > 1:
                raise Exception(f"Found more than one resource({', '.join([navigate_value(i, ['id']) for i in v])})")

            if len(v) == 1:
                resource = update_properties(module, {"read": v[0]}, None)
                module.params["id"] = navigate_value(resource, ["id"])

        result = {}
        changed = False
        if module.params["state"] == "present":
            if resource is None:
                if not module.check_mode:
                    resource = create(config)
                changed = True

            result = resource
        else:
            if resource:
                if not module.check_mode:
                    delete(config)
                changed = True

    except Exception as ex:
        module.fail_json(msg=str(ex))

    else:
        result["changed"] = changed
        module.exit_json(**result)


def user_input_parameters(module):
    return {
        "destination": module.params.get("destination"),
        "next_hop": module.params.get("next_hop"),
        "type": module.params.get("type"),
        "vpc_id": module.params.get("vpc_id"),
        "id": module.params.get("id"),
    }


def create(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")
    opts = user_input_parameters(module)

    params = build_create_parameters(opts)
    r = send_create_request(module, params, client)
    module.params["id"] = navigate_value(r, ["route", "id"])

    result = update_properties(module, {"read": fill_resp_body(r)}, None)
    return result


def delete(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")

    send_delete_request(module, None, client)


def get_resource_by_id(config, exclude_output=False):
    module = config.module
    client = config.client(get_region(module), "network", "project")

    res = {}

    r = send_read_request(module, client)
    res["read"] = fill_resp_body(r)

    result = update_properties(module, res, None, exclude_output)
    return result


def _build_query_link(opts):
    query_params = []

    v = navigate_value(opts, ["type"])
    if v:
        query_params.append(f"type={v}")

    v = navigate_value(opts, ["destination"])
    if v:
        query_params.append(f"destination={v}")

    v = navigate_value(opts, ["vpc_id"])
    if v:
        query_params.append(f"vpc_id={v}")

    query_link = "?marker={marker}&limit=10"
    if query_params:
        query_link += f"&{'&'.join(query_params)}"

    return query_link


def search_resource(config):
    module = config.module
    client = config.client(get_region(module), "network", "project")
    opts = user_input_parameters(module)
    identity_obj = _build_identity_object(opts)
    query_link = _build_query_link(opts)
    link = f"v2.0/vpc/routes{query_link}"

    result = []
    p = {"marker": ""}
    while True:
        url = link.format(**p)
        r = send_list_request(module, client, url)
        if not r:
            break

        for item in r:
            item = fill_list_resp_body(item)
            if not are_different_dicts(identity_obj, item):
                result.append(item)

        if len(result) > 1:
            break

        p["marker"] = r[-1].get("id")

    return result


def build_create_parameters(opts):
    params = dict()

    v = navigate_value(opts, ["destination"], None)
    if not is_empty_value(v):
        params["destination"] = v

    v = navigate_value(opts, ["next_hop"], None)
    if not is_empty_value(v):
        params["nexthop"] = v

    v = navigate_value(opts, ["type"], None)
    if not is_empty_value(v):
        params["type"] = v

    v = navigate_value(opts, ["vpc_id"], None)
    if not is_empty_value(v):
        params["vpc_id"] = v

    if not params:
        return params

    params = {"route": params}

    return params


def send_create_request(module, params, client):
    url = "v2.0/vpc/routes"
    try:
        r = client.post(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_route): error running api(create), error: {ex}"
        module.fail_json(msg=msg)

    return r


def send_delete_request(module, params, client):
    url = build_path(module, "v2.0/vpc/routes/{id}")

    try:
        r = client.delete(url, params)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_route): error running api(delete), error: {ex}"
        module.fail_json(msg=msg)

    return r


def send_read_request(module, client):
    url = build_path(module, "v2.0/vpc/routes/{id}")

    r = None
    try:
        r = client.get(url)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_route): error running api(read), error: {ex}"
        module.fail_json(msg=msg)

    return navigate_value(r, ["route"], None)


def fill_resp_body(body):
    result = dict()

    result["destination"] = body.get("destination")

    result["id"] = body.get("id")

    result["nexthop"] = body.get("nexthop")

    result["type"] = body.get("type")

    result["vpc_id"] = body.get("vpc_id")

    return result


def update_properties(module, response, array_index, exclude_output=False):
    r = user_input_parameters(module)

    v = navigate_value(response, ["read", "destination"], array_index)
    r["destination"] = v

    v = navigate_value(response, ["read", "nexthop"], array_index)
    r["next_hop"] = v

    v = navigate_value(response, ["read", "type"], array_index)
    r["type"] = v

    v = navigate_value(response, ["read", "vpc_id"], array_index)
    r["vpc_id"] = v

    v = navigate_value(response, ["read", "id"], array_index)
    r["id"] = v

    return r


def send_list_request(module, client, url):
    r = None
    try:
        r = client.get(url)
    except HwcClientException as ex:
        msg = f"module(hwc_vpc_route): error running api(list), error: {ex}"
        module.fail_json(msg=msg)

    return navigate_value(r, ["routes"], None)


def _build_identity_object(all_opts):
    result = dict()

    v = navigate_value(all_opts, ["destination"], None)
    result["destination"] = v

    v = navigate_value(all_opts, ["id"], None)
    result["id"] = v

    v = navigate_value(all_opts, ["next_hop"], None)
    result["nexthop"] = v

    v = navigate_value(all_opts, ["type"], None)
    result["type"] = v

    v = navigate_value(all_opts, ["vpc_id"], None)
    result["vpc_id"] = v

    return result


def fill_list_resp_body(body):
    result = dict()

    result["destination"] = body.get("destination")

    result["id"] = body.get("id")

    result["nexthop"] = body.get("nexthop")

    result["type"] = body.get("type")

    result["vpc_id"] = body.get("vpc_id")

    return result


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2023, Gaudenz Steinlin <gaudenz.steinlin@cloudscale.ch>
# Copyright: (c) 2023, Kenneth Joss <kenneth.joss@cloudscale.ch>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: load_balancer_pool_member
short_description: Manages load balancer pool members on the cloudscale.ch IaaS service
description:
  - Get, create, update, delete pool members on the cloudscale.ch IaaS service.
notes:
  - If I(uuid) option is provided, it takes precedence over I(name) for pool member selection. This allows to update the member's name.
  - If no I(uuid) option is provided, I(name) is used for pool member selection. If more than one load balancer with this name exists, execution is aborted.
author:
  - Gaudenz Steinlin (@gaudenz)
  - Kenneth Joss (@k-304)
version_added: "2.3.0"
options:
  state:
    description:
      - State of the load balancer pool member.
    choices: [ present, absent ]
    default: present
    type: str
  name:
    description:
      - Name of the load balancer pool member.
      - Either I(name) or I(uuid) are required.
    type: str
  uuid:
    description:
      - UUID of the load balancer.
      - Either I(name) or I(uuid) are required.
    type: str
  load_balancer_pool:
    description:
      - UUID of the load balancer pool.
    type: str
  enabled:
    description:
      - Pool member will not receive traffic if false. Default is true.
    default: true
    type: bool
  protocol_port:
    description:
      - The port to which actual traffic is sent.
    type: int
  monitor_port:
    description:
      - The port to which health monitor checks are sent.
      - If not specified, protocol_port will be used. Default is null.
    default: null
    type: int
  address:
    description:
      - The IP address to which traffic is sent.
    type: str
  subnet:
    description:
      - The subnet of the address must be specified here.
    type: str
  tags:
    description:
      - Tags assosiated with the load balancer. Set this to C({}) to clear any tags.
    type: dict
extends_documentation_fragment: cloudscale_ch.cloud.api_parameters
'''

EXAMPLES = '''
# Create a pool member for a load balancer pool using registered variables
- name: Create a load balancer pool
  cloudscale_ch.cloud.load_balancer_pool:
    name: 'swimming-pool'
    load_balancer: '514064c2-cfd4-4b0c-8a4b-c68c552ff84f'
    algorithm: 'round_robin'
    protocol: 'tcp'
    tags:
      project: ansible-test
      stage: production
      sla: 24-7
    api_token: xxxxxx
  register: load_balancer_pool

- name: Create a load balancer pool member
  cloudscale_ch.cloud.load_balancer_pool_member:
    name: 'my-shiny-swimming-pool-member'
    load_balancer_pool: '{{ load_balancer_pool.uuid }}'
    enabled: true
    protocol_port: 8080
    monitor_port: 8081
    subnet: '70d282ab-2a01-4abb-ada5-34e56a5a7eee'
    address: '172.16.0.100'
    tags:
      project: ansible-test
      stage: production
      sla: 24-7
    api_token: xxxxxx

# Get load balancer pool member facts by name
- name: Get facts of a load balancer pool member by name
  cloudscale_ch.cloud.load_balancer_pool_member:
    name: 'my-shiny-swimming-pool-member'
    api_token: xxxxxx
'''

RETURN = '''
href:
  description: API URL to get details about this load balancer
  returned: success when not state == absent
  type: str
  sample: https://api.cloudscale.ch/v1/load-balancers/pools/20a7eb11-3e17-4177-b46d-36e13b101d1c/members/b9991773-857d-47f6-b20b-0a03709529a9
uuid:
  description: The unique identifier for this load balancer pool member
  returned: success
  type: str
  sample: cfde831a-4e87-4a75-960f-89b0148aa2cc
name:
  description: The display name of the load balancer pool member
  returned: success
  type: str
  sample: web-lb-pool
enabled:
  description: THe status of the load balancer pool member
  returned: success
  type: bool
  sample: true
created_at:
  description: The creation date and time of the load balancer pool member
  returned: success when not state == absent
  type: str
  sample: "2023-02-07T15:32:02.308041Z"
pool:
  description: The pool of the pool member
  returned: success
  type: dict
  sample: {
            "href": "https://api.cloudscale.ch/v1/load-balancers/pools/20a7eb11-3e17-4177-b46d-36e13b101d1c",
            "uuid": "20a7eb11-3e17-4177-b46d-36e13b101d1c",
            "name": "web-lb-pool"
            }
protocol_port:
  description: The port to which actual traffic is sent
  returned: success
  type: int
  sample: 8080
monitor_port:
  description: The port to which health monitor checks are sent
  returned: success
  type: int
  sample: 8081
address:
  description: The IP address to which traffic is sent
  returned: success
  type: str
  sample: 10.11.12.3
subnet:
  description: The subnet in a private network in which address is located
  returned: success
  type: dict
  sample: {
            "href": "https://api.cloudscale.ch/v1/subnets/70d282ab-2a01-4abb-ada5-34e56a5a7eee",
            "uuid": "70d282ab-2a01-4abb-ada5-34e56a5a7eee",
            "cidr": "10.11.12.0/24"
            }
monitor_status:
  description: The status of the pool's health monitor check for this member
  returned: success
  type: str
  sample: up
tags:
  description: Tags assosiated with the load balancer
  returned: success
  type: dict
  sample: { 'project': 'my project' }
'''

from ansible.module_utils.basic import AnsibleModule
from ..module_utils.api import (
    AnsibleCloudscaleBase,
    cloudscale_argument_spec,
)

ALLOWED_STATES = ('present',
                  'absent',
                  )


class AnsibleCloudscaleLoadBalancerPoolMember(AnsibleCloudscaleBase):

    def __init__(self, module):
        super(AnsibleCloudscaleLoadBalancerPoolMember, self).__init__(
            module,
            resource_name='load-balancers/pools/%s/members' % module.params['load_balancer_pool'],
            resource_create_param_keys=[
                'name',
                'enabled',
                'protocol_port',
                'monitor_port',
                'address',
                'subnet',
                'tags',
            ],
            resource_update_param_keys=[
                'name',
                'enabled',
                'tags',
            ],
        )

    def query(self):
        # Initialize
        self._resource_data = self.init_resource()

        # Query by UUID
        uuid = self._module.params[self.resource_key_uuid]
        if uuid is not None:

            # network id case
            if "/" in uuid:
                uuid = uuid.split("/")[0]

            resource = self._get('%s/%s' % (self.resource_name, uuid))
            if resource:
                self._resource_data = resource
                self._resource_data['state'] = "present"

        # Query by name
        else:
            name = self._module.params[self.resource_key_name]

            # Resource has no name field, we use a defined tag as name
            if self.use_tag_for_name:
                resources = self._get('%s?tag:%s=%s' % (self.resource_name, self.resource_name_tag, name))
            else:
                resources = self._get('%s' % self.resource_name)

            matching = []
            if resources is None:
                self._module.fail_json(
                    msg="The load balancer pool %s does not exist."
                        % (self.resource_name,)
                )
            for resource in resources:
                if self.use_tag_for_name:
                    resource[self.resource_key_name] = resource['tags'].get(self.resource_name_tag)

                # Skip resource if constraints is not given e.g. in case of floating_ip the ip_version differs
                for constraint_key in self.query_constraint_keys:
                    if self._module.params[constraint_key] is not None:
                        if constraint_key == 'zone':
                            resource_value = resource['zone']['slug']
                        else:
                            resource_value = resource[constraint_key]

                        if resource_value != self._module.params[constraint_key]:
                            break
                else:
                    if resource[self.resource_key_name] == name:
                        matching.append(resource)

            # Fail on more than one resource with identical name
            if len(matching) > 1:
                self._module.fail_json(
                    msg="More than one %s resource with '%s' exists: %s. "
                        "Use the '%s' parameter to identify the resource." % (
                            self.resource_name,
                            self.resource_key_name,
                            name,
                            self.resource_key_uuid
                        )
                )
            elif len(matching) == 1:
                self._resource_data = matching[0]
                self._resource_data['state'] = "present"

        return self.pre_transform(self._resource_data)


def main():
    argument_spec = cloudscale_argument_spec()
    argument_spec.update(dict(
        name=dict(),
        uuid=dict(),
        load_balancer_pool=dict(type='str'),
        enabled=dict(type='bool', default=True),
        protocol_port=dict(type='int'),
        monitor_port=dict(type='int'),
        subnet=dict(type='str'),
        address=dict(type='str'),
        tags=dict(type='dict'),
        state=dict(default='present', choices=ALLOWED_STATES),
    ))

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=(),
        required_one_of=(('name', 'uuid'),),
        supports_check_mode=True,
    )

    cloudscale_load_balancer_pool_member = AnsibleCloudscaleLoadBalancerPoolMember(module)
    cloudscale_load_balancer_pool_member.query_constraint_keys = []

    if module.params['state'] == "absent":
        result = cloudscale_load_balancer_pool_member.absent()
    else:
        result = cloudscale_load_balancer_pool_member.present()
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# Copyright (c) 2022, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: openssl_publickey_info
short_description: Retrieve information from OpenSSL public keys in PEM format
version_added: 2.10.0
author:
  - Felix Fontein (@felixfontein)
description:
  - Provided a public key in OpenSSL PEM format, retrieve information.
  - This is a filter version of the M(community.crypto.openssl_publickey_info) module.
options:
  _input:
    description:
      - The content of the OpenSSL PEM public key.
    type: string
    required: true
seealso:
  - module: community.crypto.openssl_publickey_info
"""

EXAMPLES = r"""
---
- name: Show the type of a public key
  ansible.builtin.debug:
    msg: >-
      {{
        (
          lookup('ansible.builtin.file', '/path/to/public-key.pem')
          | community.crypto.openssl_publickey_info
        ).type
      }}
"""

RETURN = r"""
_value:
  description:
    - Information on the public key.
  type: dict
  contains:
    fingerprints:
      description:
        - Fingerprints of public key.
        - For every hash algorithm available, the fingerprint is computed.
      returned: success
      type: dict
      sample: "{'sha256': 'd4:b3:aa:6d:c8:04:ce:4e:ba:f6:29:4d:92:a3:94:b0:c2:ff:bd:bf:33:63:11:43:34:0f:51:b0:95:09:2f:63',
        'sha512': 'f7:07:4a:f0:b0:f0:e6:8b:95:5f:f9:e6:61:0a:32:68:f1..."
    type:
      description:
        - The key's type.
        - One of V(RSA), V(DSA), V(ECC), V(Ed25519), V(X25519), V(Ed448), or V(X448).
        - Will start with V(unknown) if the key type cannot be determined.
      returned: success
      type: str
      sample: RSA
    public_data:
      description:
        - Public key data. Depends on key type.
      returned: success
      type: dict
      contains:
        size:
          description:
            - Bit size of modulus (RSA) or prime number (DSA).
          type: int
          returned: When RV(_value.type=RSA) or RV(_value.type=DSA)
        modulus:
          description:
            - The RSA key's modulus.
          type: int
          returned: When RV(_value.type=RSA)
        exponent:
          description:
            - The RSA key's public exponent.
          type: int
          returned: When RV(_value.type=RSA)
        p:
          description:
            - The C(p) value for DSA.
            - This is the prime modulus upon which arithmetic takes place.
          type: int
          returned: When RV(_value.type=DSA)
        q:
          description:
            - The C(q) value for DSA.
            - This is a prime that divides C(p - 1), and at the same time the order of the subgroup of the multiplicative
              group of the prime field used.
          type: int
          returned: When RV(_value.type=DSA)
        g:
          description:
            - The C(g) value for DSA.
            - This is the element spanning the subgroup of the multiplicative group of the prime field used.
          type: int
          returned: When RV(_value.type=DSA)
        curve:
          description:
            - The curve's name for ECC.
          type: str
          returned: When RV(_value.type=ECC)
        exponent_size:
          description:
            - The maximum number of bits of a private key. This is basically the bit size of the subgroup used.
          type: int
          returned: When RV(_value.type=ECC)
        x:
          description:
            - The C(x) coordinate for the public point on the elliptic curve.
          type: int
          returned: When RV(_value.type=ECC)
        y:
          description:
            - For RV(_value.type=ECC), this is the C(y) coordinate for the public point on the elliptic curve.
            - For RV(_value.type=DSA), this is the publicly known group element whose discrete logarithm with respect to C(g)
              is the private key.
          type: int
          returned: When RV(_value.type=DSA) or RV(_value.type=ECC)
"""

import typing as t
from collections.abc import Callable

from ansible.errors import AnsibleFilterError
from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.publickey_info import (
    PublicKeyParseError,
    get_publickey_info,
)
from ansible_collections.community.crypto.plugins.plugin_utils._filter_module import (
    FilterModuleMock,
)


def openssl_publickey_info_filter(data: str | bytes) -> dict[str, t.Any]:
    """Extract information from OpenSSL PEM public key."""
    if not isinstance(data, (str, bytes)):
        raise AnsibleFilterError(
            f"The community.crypto.openssl_publickey_info input must be a text type, not {type(data)}"
        )

    module = FilterModuleMock({})
    try:
        return get_publickey_info(module=module, content=to_bytes(data))
    except PublicKeyParseError as exc:
        raise AnsibleFilterError(exc.error_message) from exc
    except OpenSSLObjectError as exc:
        raise AnsibleFilterError(str(exc)) from exc


class FilterModule:
    """Ansible jinja2 filters"""

    def filters(self) -> dict[str, Callable]:
        return {
            "openssl_publickey_info": openssl_publickey_info_filter,
        }



#!/usr/bin/python
# Copyright (c) 2025 Markus Bergholz
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: adguardhome_rewrite_info

short_description: Retrieve DNS rewrite rules from AdGuardHome

version_added: 3.3.0

description:
  - Retrieves DNS rewrite rules from AdGuardHome.
extends_documentation_fragment:
  - community.dns._adguardhome.connectivity
  - community.dns._attributes
  - community.dns._attributes.info_module
  - community.dns._attributes.idempotent_not_modify_state

author:
  - Markus Bergholz (@markuman) <markuman+spambelongstogoogle@gmail.com>
"""

EXAMPLES = r"""
- name: Get DNS rewrites from AdGuardHome
  register: rewrite
  community.dns.adguardhome_rewrite_info:
    username: admin
    password: admin
    host: https://dns.osuv.de

- name: Display the rewrite rules
  debug:
    var: rewrite
"""

RETURN = r"""
rules:
  description: The list of fetched rewrite rules.
  type: list
  elements: dict
  returned: success
  contains:
    answer:
      description: Value of the rewrite.
      type: str
      sample: 192.168.178.71
    domain:
      description: Domain of the rewrite.
      type: str
      sample: dns.osuv.de
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.dns.plugins.module_utils._adguardhome.api import (
    AdGuardHomeAPIHandler,
    create_adguardhome_argument_spec,
)


def main() -> None:
    argument_spec = create_adguardhome_argument_spec()
    module = AnsibleModule(supports_check_mode=True, **argument_spec.to_kwargs())

    adguardhome = AdGuardHomeAPIHandler(module.params, module.fail_json)

    result = {"changed": False, "rules": adguardhome.list()}

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_mqtt_settings
short_description: Resource module for organizations _wireless _mqtt _settings
description:
  - Manage operation update of the resource organizations _wireless _mqtt _settings.
  - Add new broker config for wireless MQTT.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  ble:
    description: MQTT BLE Settings for network.
    suboptions:
      allowLists:
        description: Allowed List for MAC and UUID.
        suboptions:
          macs:
            description: Allowed MAC List.
            elements: str
            type: list
          uuids:
            description: Allowed UUID List.
            elements: str
            type: list
        type: dict
      enabled:
        description: BLE Enabled.
        type: bool
      flush:
        description: BLE Flush frequency.
        suboptions:
          frequency:
            description: BLE Flush frequency in seconds. Will be between 1 and 2147483647.
              Default is 60 seconds.
            type: int
        type: dict
      hysteresis:
        description: BLE Hysteresis Settings for network.
        suboptions:
          enabled:
            description: BLE Hysteresis Enabled.
            type: bool
          threshold:
            description: BLE Threshold. Will be between 1 and 2147483647. Default
              is 1 second.
            type: int
        type: dict
      type:
        description: BLE type of clients to publish telemetry. Valid types are all,
          ibeacon, eddystone, unknown.
        type: str
    type: dict
  mqtt:
    description: MQTT Settings for network.
    suboptions:
      broker:
        description: MQTT Broker Settings for network.
        suboptions:
          name:
            description: Broker Config Name.
            type: str
        type: dict
      enabled:
        description: MQTT Enabled.
        type: bool
      messageFields:
        description: Select fields to populate in MQTT messages. Valid types are RSSI,
          AP MAC address, Client MAC address, Timestamp, Radio, Network ID, Beacon
          type, Raw payload, Client UUID, Client major value, Client minor value,
          Signal power, Band, Slot ID.
        elements: str
        type: list
      publishing:
        description: MQTT Publishing Settings.
        suboptions:
          frequency:
            description: MQTT Publishing Frequency in seconds. Will be between 1 and
              2147483647. Default is 1 second.
            type: int
          qos:
            description: MQTT Publishing QoS. Valid types are 0, 1, 2.
            type: int
        type: dict
      topic:
        description: MQTT Topic.
        type: str
    type: dict
  network:
    description: Add MQTT Settings for network.
    suboptions:
      id:
        description: Network ID.
        type: str
    type: dict
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  wifi:
    description: MQTT Wi-Fi Settings for network.
    suboptions:
      allowLists:
        description: Allowed List for MAC and UUID.
        suboptions:
          macs:
            description: Allowed MAC List.
            elements: str
            type: list
        type: dict
      enabled:
        description: Wi-Fi Enabled.
        type: bool
      flush:
        description: BLE Flush frequency.
        suboptions:
          frequency:
            description: Wi-Fi Flush frequency in seconds. Will be between 1 and 2147483647.
              Default is 60 seconds.
            type: int
        type: dict
      hysteresis:
        description: Wi-Fi Hysteresis Settings for network.
        suboptions:
          enabled:
            description: Wi-Fi Hysteresis Enabled.
            type: bool
          threshold:
            description: Wi-Fi Threshold. Will be between 1 and 2147483647. Default
              is 1 second.
            type: int
        type: dict
      type:
        description: Wi-Fi client type. Valid types are visible, associated.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless updateOrganizationWirelessMqttSettings
    description: Complete reference of the updateOrganizationWirelessMqttSettings
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-wireless-mqtt-settings
notes:
  - SDK Method used are
    wireless.Wireless.update_organization_wireless_mqtt_settings,
  - Paths used are
    put /organizations/{organizationId}/wireless/mqtt/settings,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.organizations_wireless_mqtt_settings:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    ble:
      allowLists:
        macs: []
        uuids: []
      enabled: false
      flush:
        frequency: 60
      hysteresis:
        enabled: true
        threshold: 1
      type: ibeacon
    mqtt:
      broker:
        name: My Broker
      enabled: true
      messageFields:
        - RSSI
        - AP MAC address
        - Client MAC address
        - Timestamp
        - Radio
        - Network ID
        - Beacon type
        - Raw payload
        - Client UUID
        - Client major value
        - Client minor value
        - Signal power
        - Band
        - Slot ID
      publishing:
        frequency: 1
        qos: 1
      topic: Test Topic
    network:
      id: L_1234
    organizationId: string
    wifi:
      allowLists:
        macs: []
      enabled: false
      flush:
        frequency: 60
      hysteresis:
        enabled: false
        threshold: 1
      type: associated
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "network": {
        "id": "string",
        "name": "string"
      },
      "mqtt": {
        "settingsId": "string",
        "enabled": true,
        "topic": "string",
        "messageFields": [
          "string"
        ],
        "publishing": {
          "frequency": 0,
          "qos": 0
        },
        "broker": {
          "id": "string",
          "name": "string"
        }
      },
      "ble": {
        "enabled": true,
        "type": "string",
        "flush": {
          "frequency": 0
        },
        "allowLists": {
          "uuids": [
            "string"
          ],
          "macs": [
            "string"
          ]
        },
        "hysteresis": {
          "enabled": true,
          "threshold": 0
        }
      },
      "wifi": {
        "enabled": true,
        "type": "string",
        "flush": {
          "frequency": 0
        },
        "allowLists": {
          "macs": [
            "string"
          ]
        },
        "hysteresis": {
          "enabled": true,
          "threshold": 0
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_export_access_rulebase
short_description: Retrieve the entire content of an Access Rules layer.
description:
  - Retrieve the entire content of an Access Rules layer. The reply features a list of objects; Each object in the reply may be a section of the layer,
    with all its rules in, or a rule itself, for the case of rules which are under the global section. In case a rule has an Access Layer applied on it, the
    entire content of the inline layer will be included in the reply as well.
  - All operations are performed over Web Services API.
  - Available from R82.20 Management version.
version_added: "6.9.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  name:
    description:
      - Object name. Must be unique in the domain.
    type: str
  package:
    description:
      - Name of the package.
    type: str
  show_expiration_settings:
    description:
      - Indicates whether to calculate and show "expiration date settings" field in reply.
    type: bool
  show_hits:
    description:
      - Show hitcount data.
    type: bool
  use_object_dictionary:
    description:
      - N/A
    type: bool
  hits_settings:
    description:
      - Hitcount settings, define the range if hits to show.
    type: dict
    suboptions:
      from_date:
        description:
          - Format, YYYY-MM-DD, YYYY-mm-ddThh,mm,ss.
        type: str
      target:
        description:
          - Target gateway name or UID.
        type: str
      to_date:
        description:
          - Format, YYYY-MM-DD, YYYY-mm-ddThh,mm,ss.
        type: str
  dereference_group_members:
    description:
      - Indicates whether to dereference "members" field by details level for every object in reply.
    type: bool
  show_membership:
    description:
      - Indicates whether to calculate and show "groups" field for every object in reply.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: export-access-rulebase
  cp_mgmt_export_access_rulebase:
    name: Corp-Access
"""

RETURN = """
cp_mgmt_export_access_rulebase:
  description: The checkpoint export-access-rulebase output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        name=dict(type='str'),
        package=dict(type='str'),
        show_expiration_settings=dict(type='bool'),
        show_hits=dict(type='bool'),
        use_object_dictionary=dict(type='bool'),
        hits_settings=dict(type='dict', options=dict(
            from_date=dict(type='str'),
            target=dict(type='str'),
            to_date=dict(type='str')
        )),
        dereference_group_members=dict(type='bool'),
        show_membership=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "export-access-rulebase"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: ec2_vpc_peering_info
short_description: Retrieves AWS VPC Peering details using AWS methods
version_added: 1.0.0
version_added_collection: community.aws
description:
  - Gets various details related to AWS VPC Peers
options:
  peer_connection_ids:
    description:
      - List of specific VPC peer IDs to get details for.
    type: list
    elements: str
  filters:
    description:
      - A dict of filters to apply. Each dict item consists of a filter key and a filter value.
        See U(https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeVpcPeeringConnections.html)
        for possible filters.
    type: dict
    default: {}
author:
  - Karen Cheng (@Etherdaemon)
  - Alina Buzachis (@alinabuzachis)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: List all EC2 VPC Peering Connections
  amazon.aws.ec2_vpc_peering_info:
    region: ap-southeast-2
  register: all_vpc_peers

- name: Debugging the result
  ansible.builtin.debug:
    msg: "{{ all_vpc_peers.result }}"

- name: Get details on specific EC2 VPC Peering Connection
  amazon.aws.ec2_vpc_peering_info:
    peer_connection_ids:
      - "pcx-12345678"
      - "pcx-87654321"
    region: "ap-southeast-2"
  register: all_vpc_peers

- name: Get all EC2 VPC Peering Connections with specific filters
  amazon.aws.ec2_vpc_peering_info:
    region: "ap-southeast-2"
    filters:
      status-code: ["pending-acceptance"]
  register: pending_vpc_peers
"""

RETURN = r"""
vpc_peering_connections:
  description: Details of the matching VPC peering connections.
  returned: success
  type: list
  elements: dict
  contains:
    accepter_vpc_info:
      description: Information about the VPC which accepted the connection.
      returned: success
      type: dict
      contains:
        cidr_block:
          description: The primary CIDR for the VPC.
          returned: when connection is in the accepted state.
          type: str
          sample: "10.10.10.0/23"
        cidr_block_set:
          description: A list of all CIDRs for the VPC.
          returned: when connection is in the accepted state.
          type: list
          elements: dict
          contains:
            cidr_block:
              description: A CIDR block used by the VPC.
              returned: success
              type: str
              sample: "10.10.10.0/23"
        owner_id:
          description: The AWS account that owns the VPC.
          returned: success
          type: str
          sample: "123456789012"
        peering_options:
          description: Additional peering configuration.
          returned: when connection is in the accepted state.
          type: dict
          contains:
            allow_dns_resolution_from_remote_vpc:
              description: Indicates whether a VPC can resolve public DNS hostnames to private IP addresses when queried from instances in a peer VPC.
              returned: success
              type: bool
            allow_egress_from_local_classic_link_to_remote_vpc:
              description: Indicates whether a local ClassicLink connection can communicate with the peer VPC over the VPC peering connection.
              returned: success
              type: bool
            allow_egress_from_local_vpc_to_remote_classic_link:
              description: Indicates whether a local VPC can communicate with a ClassicLink connection in the peer VPC over the VPC peering connection.
              returned: success
              type: bool
        region:
          description: The AWS region that the VPC is in.
          returned: success
          type: str
          sample: "us-east-1"
        vpc_id:
          description: The ID of the VPC
          returned: success
          type: str
          sample: "vpc-0123456789abcdef0"
    requester_vpc_info:
      description: Information about the VPC which requested the connection.
      returned: success
      type: dict
      contains:
        cidr_block:
          description: The primary CIDR for the VPC.
          returned: when connection is not in the deleted state.
          type: str
          sample: "10.10.10.0/23"
        cidr_block_set:
          description: A list of all CIDRs for the VPC.
          returned: when connection is not in the deleted state.
          type: list
          elements: dict
          contains:
            cidr_block:
              description: A CIDR block used by the VPC
              returned: success
              type: str
              sample: "10.10.10.0/23"
        owner_id:
          description: The AWS account that owns the VPC.
          returned: success
          type: str
          sample: "123456789012"
        peering_options:
          description: Additional peering configuration.
          returned: when connection is not in the deleted state.
          type: dict
          contains:
            allow_dns_resolution_from_remote_vpc:
              description: Indicates whether a VPC can resolve public DNS hostnames to private IP addresses when queried from instances in a peer VPC.
              returned: success
              type: bool
            allow_egress_from_local_classic_link_to_remote_vpc:
              description: Indicates whether a local ClassicLink connection can communicate with the peer VPC over the VPC peering connection.
              returned: success
              type: bool
            allow_egress_from_local_vpc_to_remote_classic_link:
              description: Indicates whether a local VPC can communicate with a ClassicLink connection in the peer VPC over the VPC peering connection.
              returned: success
              type: bool
        region:
          description: The AWS region that the VPC is in.
          returned: success
          type: str
          sample: "us-east-1"
        vpc_id:
          description: The ID of the VPC
          returned: success
          type: str
          sample: "vpc-0123456789abcdef0"
    status:
      description: Details of the current status of the connection.
      returned: success
      type: dict
      contains:
        code:
          description: A short code describing the status of the connection.
          returned: success
          type: str
          sample: "active"
        message:
          description: Additional information about the status of the connection.
          returned: success
          type: str
          sample: "Pending Acceptance by 123456789012"
    tags:
      description: Tags applied to the connection.
      returned: success
      type: dict
    vpc_peering_connection_id:
      description: The ID of the VPC peering connection.
      returned: success
      type: str
      sample: "pcx-0123456789abcdef0"
"""


from typing import Any
from typing import Dict
from typing import List

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import normalize_boto3_result
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_vpc_peering_connections
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list


def get_vpc_peers(client, module: AnsibleAWSModule) -> List[Dict[str, Any]]:
    params: Dict = {}
    params["Filters"] = ansible_dict_to_boto3_filter_list(module.params.get("filters"))

    if module.params.get("peer_connection_ids"):
        params["VpcPeeringConnectionIds"] = module.params.get("peer_connection_ids")

    result = describe_vpc_peering_connections(client, **params)

    return normalize_boto3_result(result)


def main():
    argument_spec = dict(
        filters=dict(default=dict(), type="dict"),
        peer_connection_ids=dict(default=None, type="list", elements="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    client = module.client("ec2")

    # Turn the boto3 result in to ansible friendly_snaked_names
    results = [camel_dict_to_snake_dict(peer) for peer in get_vpc_peers(client, module)]

    # Turn the boto3 result in to ansible friendly tag dictionary
    for peer in results:
        peer["tags"] = boto3_tag_list_to_ansible_dict(peer.get("tags", []))

    module.exit_json(vpc_peering_connections=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: elb_target_group
version_added: 1.0.0
short_description: Manage a target group for an Application or Network load balancer
description:
  - Manage an AWS Elastic Load Balancer target group. See
    U(https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-target-groups.html) or
    U(https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-target-groups.html) for details.
author:
  - "Rob White (@wimnat)"
options:
  deregistration_delay_timeout:
    description:
      - The amount time for Elastic Load Balancing to wait before changing the state of a deregistering target from draining to unused.
        The range is 0-3600 seconds.
    type: int
  deregistration_connection_termination:
    description:
      - Indicates whether the load balancer terminates connections at the end of the deregistration timeout.
      - Using this option is only supported when attaching to a Network Load Balancer (NLB).
    type: bool
    default: false
    required: false
    version_added: 3.1.0
  health_check_protocol:
    description:
      - The protocol the load balancer uses when performing health checks on targets.
    required: false
    choices: [ 'http', 'https', 'tcp', 'tls', 'udp', 'tcp_udp', 'HTTP', 'HTTPS', 'TCP', 'TLS', 'UDP', 'TCP_UDP']
    type: str
  health_check_port:
    description:
      - The port the load balancer uses when performing health checks on targets.
        Can be set to 'traffic-port' to match target port.
      - When not defined will default to the port on which each target receives traffic from the load balancer.
    required: false
    type: str
  health_check_path:
    description:
      - The ping path that is the destination on the targets for health checks. The path must be defined in order to set a health check.
      - Requires the I(health_check_protocol) parameter to be set.
    required: false
    type: str
  health_check_interval:
    description:
      - The approximate amount of time, in seconds, between health checks of an individual target.
    required: false
    type: int
  health_check_timeout:
    description:
      - The amount of time, in seconds, during which no response from a target means a failed health check.
    required: false
    type: int
  healthy_threshold_count:
    description:
      - The number of consecutive health checks successes required before considering an unhealthy target healthy.
    required: false
    type: int
  modify_targets:
    description:
      - Whether or not to alter existing targets in the group to match what is passed with the module
    required: false
    default: true
    type: bool
  name:
    description:
      - The name of the target group.
    required: true
    type: str
  port:
    description:
      - The port on which the targets receive traffic. This port is used unless you specify a port override when registering the target.
      - Required when I(state) is C(present) and I(target_type) is C(instance), C(ip), or C(alb).
    required: false
    type: int
  protocol:
    description:
      - The protocol to use for routing traffic to the targets.
      - Required when I(state) is C(present) and I(target_type) is C(instance), C(ip), or C(alb).
    required: false
    choices: [ 'http', 'https', 'tcp', 'tls', 'udp', 'tcp_udp', 'HTTP', 'HTTPS', 'TCP', 'TLS', 'UDP', 'TCP_UDP']
    type: str
  protocol_version:
    description:
      - Specifies protocol version.
      - The protocol_version parameter is immutable and cannot be changed when updating an elb_target_group.
    required: false
    choices: ['GRPC', 'HTTP1', 'HTTP2']
    type: str
    version_added: 5.1.0
  state:
    description:
      - Create or destroy the target group.
    required: true
    choices: [ 'present', 'absent' ]
    type: str
  stickiness_enabled:
    description:
      - Indicates whether sticky sessions are enabled.
    type: bool
  stickiness_lb_cookie_duration:
    description:
      - The time period, in seconds, during which requests from a client should be routed to the same target. After this time period expires, the load
        balancer-generated cookie is considered stale. The range is 1 second to 1 week (604800 seconds).
    type: int
  stickiness_app_cookie_duration:
    description:
      - The time period, in seconds, during which requests from a client
        should be routed to the same target. After this time period expires,
        the application-generated cookie is considered stale. The range is 1 second to 1 week (604800 seconds).
    type: int
    version_added: 1.5.0
  stickiness_app_cookie_name:
    description:
      - The name of the application cookie. Required if I(stickiness_type=app_cookie).
    type: str
    version_added: 1.5.0
  stickiness_type:
    description:
      - The type of sticky sessions.
      - Valid values are C(lb_cookie), C(app_cookie) or C(source_ip).
      - If not set AWS will default to C(lb_cookie) for Application Load Balancers or C(source_ip) for Network Load Balancers.
    type: str
  load_balancing_algorithm_type:
    description:
      - The type of load balancing algorithm to use.
      - Changing the load balancing algorithm is only supported when used with Application Load Balancers (ALB).
      - If not set AWS will default to C(round_robin).
    choices: ['round_robin', 'least_outstanding_requests']
    type: str
    version_added: 3.2.0
  successful_response_codes:
    description:
      - The HTTP codes to use when checking for a successful response from a target.
      - Accepts multiple values (for example, "200,202") or a range of values (for example, "200-299").
      - Requires the I(health_check_protocol) parameter to be set.
    required: false
    type: str
  target_type:
    description:
      - The type of target that you must specify when registering targets with this target group. The possible values are
        C(instance) (targets are specified by instance ID), C(ip) (targets are specified by IP address), C(lambda) (target is specified by ARN),
        or C(alb) (target is specified by ARN).
        Note that you can't specify targets for a target group using more than one type. Target types lambda and alb only accept one target. When more than
        one target is specified, only the first one is used. All additional targets are ignored.
        If the target type is ip, specify IP addresses from the subnets of the virtual private cloud (VPC) for the target
        group, the RFC 1918 range (10.0.0.0/8, 172.16.0.0/12, and 192.168.0.0/16), and the RFC 6598 range (100.64.0.0/10).
        You can't specify publicly routable IP addresses.
      - The default behavior is C(instance).
    required: false
    choices: ['instance', 'ip', 'lambda', 'alb']
    type: str
  targets:
    description:
      - A list of targets to assign to the target group. This parameter defaults to an empty list. Unless you set the 'modify_targets' parameter then
        all existing targets will be removed from the group. The list should be an Id and a Port parameter. See the Examples for detail.
    required: false
    type: list
    elements: dict
  unhealthy_threshold_count:
    description:
      - The number of consecutive health check failures required before considering a target unhealthy.
    required: false
    type: int
  vpc_id:
    description:
      - The identifier of the virtual private cloud (VPC).
      - Required when I(state) is C(present) and I(target_type) is C(instance), C(ip), or C(alb).
    required: false
    type: str
  preserve_client_ip_enabled:
    description:
      - Indicates whether client IP preservation is enabled.
      - The default is disabled if the target group type is C(ip) address and the target group protocol is C(tcp) or C(tls).
        Otherwise, the default is enabled. Client IP preservation cannot be disabled for C(udp) and C(tcp_udp) target groups.
      - I(preserve_client_ip_enabled) is supported only by Network Load Balancers.
    type: bool
    required: false
    version_added: 2.1.0
  proxy_protocol_v2_enabled:
    description:
      - Indicates whether Proxy Protocol version 2 is enabled.
      - The value is C(true) or C(false).
      - I(proxy_protocol_v2_enabled) is supported only by Network Load Balancers.
    type: bool
    required: false
    version_added: 2.1.0
  wait:
    description:
      - Whether or not to wait for the target group.
    type: bool
    default: false
  wait_timeout:
    description:
      - The time to wait for the target group.
    default: 200
    type: int

notes:
  - Once a target group has been created, only its health check can then be modified using subsequent calls

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create a target group with a default health check
  community.aws.elb_target_group:
    name: mytargetgroup
    protocol: http
    port: 80
    vpc_id: vpc-01234567
    state: present

- name: Create a target group with protocol_version 'GRPC'
  community.aws.elb_target_group:
    name: mytargetgroup
    protocol: http
    port: 80
    vpc_id: vpc-01234567
    protocol_version: GRPC
    state: present

- name: Modify the target group with a custom health check
  community.aws.elb_target_group:
    name: mytargetgroup
    protocol: http
    port: 80
    vpc_id: vpc-01234567
    health_check_protocol: http
    health_check_path: /health_check
    health_check_port: 80
    successful_response_codes: 200
    health_check_interval: 15
    health_check_timeout: 3
    healthy_threshold_count: 4
    unhealthy_threshold_count: 3
    state: present

- name: Delete a target group
  community.aws.elb_target_group:
    name: mytargetgroup
    state: absent

- name: Create a target group with instance targets
  community.aws.elb_target_group:
    name: mytargetgroup
    protocol: http
    port: 81
    vpc_id: vpc-01234567
    health_check_protocol: http
    health_check_path: /
    successful_response_codes: "200,250-260"
    targets:
      - Id: i-01234567
        Port: 80
      - Id: i-98765432
        Port: 80
    state: present
    wait_timeout: 200
    wait: true

- name: Create a target group with IP address targets
  community.aws.elb_target_group:
    name: mytargetgroup
    protocol: http
    port: 81
    vpc_id: vpc-01234567
    health_check_protocol: http
    health_check_path: /
    successful_response_codes: "200,250-260"
    target_type: ip
    targets:
      - Id: 10.0.0.10
        Port: 80
        AvailabilityZone: all
      - Id: 10.0.0.20
        Port: 80
    state: present
    wait_timeout: 200
    wait: true

# Using lambda as targets require that the target group
# itself is allow to invoke the lambda function.
# therefore you need first to create an empty target group
# to receive its arn, second, allow the target group
# to invoke the lambda function and third, add the target
# to the target group
- name: first, create empty target group
  community.aws.elb_target_group:
    name: my-lambda-targetgroup
    target_type: lambda
    state: present
    modify_targets: false
  register: out

- name: second, allow invoke of the lambda
  community.aws.lambda_policy:
    state: "{{ state | default('present') }}"
    function_name: my-lambda-function
    statement_id: someID
    action: lambda:InvokeFunction
    principal: elasticloadbalancing.amazonaws.com
    source_arn: "{{ out.target_group_arn }}"

- name: third, add target
  community.aws.elb_target_group:
    name: my-lambda-targetgroup
    target_type: lambda
    state: present
    targets:
      - Id: arn:aws:lambda:eu-central-1:123456789012:function:my-lambda-function
"""

RETURN = r"""
deregistration_delay_timeout_seconds:
    description: The amount time for Elastic Load Balancing to wait before changing the state of a deregistering target from draining to unused.
    returned: when state present
    type: int
    sample: 300
deregistration_connection_termination:
    description: Indicates whether the load balancer terminates connections at the end of the deregistration timeout.
    returned: when state present
    type: bool
    sample: True
health_check_interval_seconds:
    description: The approximate amount of time, in seconds, between health checks of an individual target.
    returned: when state present
    type: int
    sample: 30
health_check_path:
    description: The destination for the health check request.
    returned: when state present
    type: str
    sample: /index.html
health_check_port:
    description: The port to use to connect with the target.
    returned: when state present
    type: str
    sample: traffic-port
health_check_protocol:
    description: The protocol to use to connect with the target.
    returned: when state present
    type: str
    sample: HTTP
health_check_timeout_seconds:
    description: The amount of time, in seconds, during which no response means a failed health check.
    returned: when state present
    type: int
    sample: 5
healthy_threshold_count:
    description: The number of consecutive health checks successes required before considering an unhealthy target healthy.
    returned: when state present
    type: int
    sample: 5
load_balancer_arns:
    description: The Amazon Resource Names (ARN) of the load balancers that route traffic to this target group.
    returned: when state present
    type: list
    sample: []
matcher:
    description: The HTTP codes to use when checking for a successful response from a target.
    returned: when state present
    type: dict
    sample: {
        "http_code": "200"
    }
port:
    description: The port on which the targets are listening.
    returned: when state present
    type: int
    sample: 80
protocol:
    description: The protocol to use for routing traffic to the targets.
    returned: when state present
    type: str
    sample: HTTP
stickiness_enabled:
    description: Indicates whether sticky sessions are enabled.
    returned: when state present
    type: bool
    sample: true
stickiness_lb_cookie_duration_seconds:
    description: The time period, in seconds, during which requests from a client should be routed to the same target.
    returned: when state present
    type: int
    sample: 86400
stickiness_type:
    description: The type of sticky sessions.
    returned: when state present
    type: str
    sample: lb_cookie
load_balancing_algorithm_type:
    description: The type load balancing algorithm used.
    returned: when state present
    type: str
    version_added: 3.2.0
    sample: least_outstanding_requests
tags:
    description: The tags attached to the target group.
    returned: when state present
    type: dict
    sample: "{
        'Tag': 'Example'
    }"
target_group_arn:
    description: The Amazon Resource Name (ARN) of the target group.
    returned: when state present
    type: str
    sample: "arn:aws:elasticloadbalancing:ap-southeast-2:123456789012:targetgroup/mytargetgroup/aabbccddee0044332211"
target_group_name:
    description: The name of the target group.
    returned: when state present
    type: str
    sample: mytargetgroup
unhealthy_threshold_count:
    description: The number of consecutive health check failures required before considering the target unhealthy.
    returned: when state present
    type: int
    sample: 2
vpc_id:
    description: The ID of the VPC for the targets.
    returned: when state present
    type: str
    sample: vpc-0123456
"""

import time

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def get_tg_attributes(connection, module, tg_arn):
    try:
        _attributes = connection.describe_target_group_attributes(TargetGroupArn=tg_arn, aws_retry=True)
        tg_attributes = boto3_tag_list_to_ansible_dict(_attributes["Attributes"])
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't get target group attributes")

    # Replace '.' with '_' in attribute key names to make it more Ansible friendly
    return dict((k.replace(".", "_"), v) for k, v in tg_attributes.items())


def get_target_group_tags(connection, module, target_group_arn):
    try:
        _tags = connection.describe_tags(ResourceArns=[target_group_arn], aws_retry=True)
        return _tags["TagDescriptions"][0]["Tags"]
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Couldn't get target group tags")


def get_target_group(connection, module, retry_missing=False):
    extra_codes = ["TargetGroupNotFound"] if retry_missing else []
    try:
        target_group_paginator = connection.get_paginator("describe_target_groups").paginate(
            Names=[module.params.get("name")]
        )
        jittered_retry = AWSRetry.jittered_backoff(retries=10, catch_extra_error_codes=extra_codes)
        result = jittered_retry(target_group_paginator.build_full_result)()
    except is_boto3_error_code("TargetGroupNotFound"):
        return None
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Couldn't get target group")

    return result["TargetGroups"][0]


def wait_for_status(connection, module, target_group_arn, targets, status):
    polling_increment_secs = 5
    max_retries = module.params.get("wait_timeout") // polling_increment_secs
    status_achieved = False

    for x in range(0, max_retries):
        try:
            response = connection.describe_target_health(
                TargetGroupArn=target_group_arn, Targets=targets, aws_retry=True
            )
            if response["TargetHealthDescriptions"][0]["TargetHealth"]["State"] == status:
                status_achieved = True
                break
            time.sleep(polling_increment_secs)
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't describe target health")

    result = response
    return status_achieved, result


def create_or_update_attributes(connection, module, target_group, new_target_group):
    changed = False
    target_type = module.params.get("target_type")
    deregistration_delay_timeout = module.params.get("deregistration_delay_timeout")
    deregistration_connection_termination = module.params.get("deregistration_connection_termination")
    stickiness_enabled = module.params.get("stickiness_enabled")
    stickiness_lb_cookie_duration = module.params.get("stickiness_lb_cookie_duration")
    stickiness_type = module.params.get("stickiness_type")
    stickiness_app_cookie_duration = module.params.get("stickiness_app_cookie_duration")
    stickiness_app_cookie_name = module.params.get("stickiness_app_cookie_name")
    preserve_client_ip_enabled = module.params.get("preserve_client_ip_enabled")
    proxy_protocol_v2_enabled = module.params.get("proxy_protocol_v2_enabled")
    load_balancing_algorithm_type = module.params.get("load_balancing_algorithm_type")

    # Now set target group attributes
    update_attributes = []

    # Get current attributes
    current_tg_attributes = get_tg_attributes(connection, module, target_group["TargetGroupArn"])

    if deregistration_delay_timeout is not None:
        if str(deregistration_delay_timeout) != current_tg_attributes["deregistration_delay_timeout_seconds"]:
            update_attributes.append(
                {"Key": "deregistration_delay.timeout_seconds", "Value": str(deregistration_delay_timeout)}
            )
    if deregistration_connection_termination is not None:
        if (
            deregistration_connection_termination
            and current_tg_attributes.get("deregistration_delay_connection_termination_enabled") != "true"
        ):
            update_attributes.append({"Key": "deregistration_delay.connection_termination.enabled", "Value": "true"})
    if stickiness_enabled is not None:
        if stickiness_enabled and current_tg_attributes["stickiness_enabled"] != "true":
            update_attributes.append({"Key": "stickiness.enabled", "Value": "true"})
    if stickiness_lb_cookie_duration is not None:
        if str(stickiness_lb_cookie_duration) != current_tg_attributes["stickiness_lb_cookie_duration_seconds"]:
            update_attributes.append(
                {"Key": "stickiness.lb_cookie.duration_seconds", "Value": str(stickiness_lb_cookie_duration)}
            )
    if stickiness_type is not None:
        if stickiness_type != current_tg_attributes.get("stickiness_type"):
            update_attributes.append({"Key": "stickiness.type", "Value": stickiness_type})
    if stickiness_app_cookie_name is not None:
        if stickiness_app_cookie_name != current_tg_attributes.get("stickiness_app_cookie_name"):
            update_attributes.append(
                {"Key": "stickiness.app_cookie.cookie_name", "Value": str(stickiness_app_cookie_name)}
            )
    if stickiness_app_cookie_duration is not None:
        if str(stickiness_app_cookie_duration) != current_tg_attributes["stickiness_app_cookie_duration_seconds"]:
            update_attributes.append(
                {"Key": "stickiness.app_cookie.duration_seconds", "Value": str(stickiness_app_cookie_duration)}
            )
    if preserve_client_ip_enabled is not None:
        if target_type not in ("udp", "tcp_udp"):
            if str(preserve_client_ip_enabled).lower() != current_tg_attributes.get("preserve_client_ip_enabled"):
                update_attributes.append(
                    {"Key": "preserve_client_ip.enabled", "Value": str(preserve_client_ip_enabled).lower()}
                )
    if proxy_protocol_v2_enabled is not None:
        if str(proxy_protocol_v2_enabled).lower() != current_tg_attributes.get("proxy_protocol_v2_enabled"):
            update_attributes.append(
                {"Key": "proxy_protocol_v2.enabled", "Value": str(proxy_protocol_v2_enabled).lower()}
            )
    if load_balancing_algorithm_type is not None:
        if str(load_balancing_algorithm_type) != current_tg_attributes["load_balancing_algorithm_type"]:
            update_attributes.append(
                {"Key": "load_balancing.algorithm.type", "Value": str(load_balancing_algorithm_type)}
            )

    if update_attributes:
        try:
            connection.modify_target_group_attributes(
                TargetGroupArn=target_group["TargetGroupArn"], Attributes=update_attributes, aws_retry=True
            )
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            # Something went wrong setting attributes. If this target group was created during this task, delete it to leave a consistent state
            if new_target_group:
                connection.delete_target_group(TargetGroupArn=target_group["TargetGroupArn"], aws_retry=True)
            module.fail_json_aws(e, msg="Couldn't delete target group")

    return changed


def create_or_update_target_group(connection, module):
    changed = False
    new_target_group = False
    params = dict()
    target_type = module.params.get("target_type")
    params["Name"] = module.params.get("name")
    params["TargetType"] = target_type
    if target_type != "lambda":
        params["Protocol"] = module.params.get("protocol").upper()
        if module.params.get("protocol_version") is not None:
            params["ProtocolVersion"] = module.params.get("protocol_version")
        params["Port"] = module.params.get("port")
        params["VpcId"] = module.params.get("vpc_id")
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")

    health_option_keys = [
        "health_check_path",
        "health_check_protocol",
        "health_check_interval",
        "health_check_timeout",
        "healthy_threshold_count",
        "unhealthy_threshold_count",
        "successful_response_codes",
    ]
    health_options = any(module.params[health_option_key] is not None for health_option_key in health_option_keys)

    # Set health check if anything set
    if health_options:
        if module.params.get("health_check_protocol") is not None:
            params["HealthCheckProtocol"] = module.params.get("health_check_protocol").upper()

        if module.params.get("health_check_port") is not None:
            params["HealthCheckPort"] = module.params.get("health_check_port")

        if module.params.get("health_check_interval") is not None:
            params["HealthCheckIntervalSeconds"] = module.params.get("health_check_interval")

        if module.params.get("health_check_timeout") is not None:
            params["HealthCheckTimeoutSeconds"] = module.params.get("health_check_timeout")

        if module.params.get("healthy_threshold_count") is not None:
            params["HealthyThresholdCount"] = module.params.get("healthy_threshold_count")

        if module.params.get("unhealthy_threshold_count") is not None:
            params["UnhealthyThresholdCount"] = module.params.get("unhealthy_threshold_count")

        # Only need to check response code and path for http(s) health checks
        protocol = module.params.get("health_check_protocol")
        if protocol is not None and protocol.upper() in ["HTTP", "HTTPS"]:
            if module.params.get("health_check_path") is not None:
                params["HealthCheckPath"] = module.params.get("health_check_path")

            if module.params.get("successful_response_codes") is not None:
                params["Matcher"] = {}
                code_key = "HttpCode"
                protocol_version = module.params.get("protocol_version")
                if protocol_version is not None and protocol_version.upper() == "GRPC":
                    code_key = "GrpcCode"
                params["Matcher"][code_key] = module.params.get("successful_response_codes")

    # Get target group
    target_group = get_target_group(connection, module)

    if target_group:
        diffs = [param for param in ("Port", "Protocol", "VpcId") if target_group.get(param) != params.get(param)]
        if diffs:
            module.fail_json(msg=f"Cannot modify {', '.join(diffs)} parameter(s) for a target group")
        # Target group exists so check health check parameters match what has been passed
        health_check_params = dict()

        # Modify health check if anything set
        if health_options:
            # Health check protocol
            if "HealthCheckProtocol" in params and target_group["HealthCheckProtocol"] != params["HealthCheckProtocol"]:
                health_check_params["HealthCheckProtocol"] = params["HealthCheckProtocol"]

            # Health check port
            if "HealthCheckPort" in params and target_group["HealthCheckPort"] != params["HealthCheckPort"]:
                health_check_params["HealthCheckPort"] = params["HealthCheckPort"]

            # Health check interval
            if (
                "HealthCheckIntervalSeconds" in params
                and target_group["HealthCheckIntervalSeconds"] != params["HealthCheckIntervalSeconds"]
            ):
                health_check_params["HealthCheckIntervalSeconds"] = params["HealthCheckIntervalSeconds"]

            # Health check timeout
            if (
                "HealthCheckTimeoutSeconds" in params
                and target_group["HealthCheckTimeoutSeconds"] != params["HealthCheckTimeoutSeconds"]
            ):
                health_check_params["HealthCheckTimeoutSeconds"] = params["HealthCheckTimeoutSeconds"]

            # Healthy threshold
            if (
                "HealthyThresholdCount" in params
                and target_group["HealthyThresholdCount"] != params["HealthyThresholdCount"]
            ):
                health_check_params["HealthyThresholdCount"] = params["HealthyThresholdCount"]

            # Unhealthy threshold
            if (
                "UnhealthyThresholdCount" in params
                and target_group["UnhealthyThresholdCount"] != params["UnhealthyThresholdCount"]
            ):
                health_check_params["UnhealthyThresholdCount"] = params["UnhealthyThresholdCount"]

            # Only need to check response code and path for http(s) health checks
            if target_group["HealthCheckProtocol"] in ["HTTP", "HTTPS"]:
                # Health check path
                if "HealthCheckPath" in params and target_group["HealthCheckPath"] != params["HealthCheckPath"]:
                    health_check_params["HealthCheckPath"] = params["HealthCheckPath"]

                # Matcher (successful response codes)
                # TODO: required and here?
                if "Matcher" in params:
                    code_key = "HttpCode"
                    if target_group.get("ProtocolVersion") == "GRPC":
                        code_key = "GrpcCode"
                    current_matcher_list = target_group["Matcher"][code_key].split(",")
                    requested_matcher_list = params["Matcher"][code_key].split(",")
                    if set(current_matcher_list) != set(requested_matcher_list):
                        health_check_params["Matcher"] = {}
                        health_check_params["Matcher"][code_key] = ",".join(requested_matcher_list)

            try:
                if health_check_params:
                    connection.modify_target_group(
                        TargetGroupArn=target_group["TargetGroupArn"], aws_retry=True, **health_check_params
                    )
                    changed = True
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Couldn't update target group")

        # Do we need to modify targets?
        if module.params.get("modify_targets"):
            # get list of current target instances. I can't see anything like a describe targets in the doco so
            # describe_target_health seems to be the only way to get them
            try:
                current_targets = connection.describe_target_health(
                    TargetGroupArn=target_group["TargetGroupArn"], aws_retry=True
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Couldn't get target group health")

            if module.params.get("targets"):
                if target_type != "lambda":
                    params["Targets"] = module.params.get("targets")

                    # Correct type of target ports
                    for target in params["Targets"]:
                        target["Port"] = int(target.get("Port", module.params.get("port")))

                    current_instance_ids = []

                    for instance in current_targets["TargetHealthDescriptions"]:
                        current_instance_ids.append(instance["Target"]["Id"])

                    new_instance_ids = []
                    for instance in params["Targets"]:
                        new_instance_ids.append(instance["Id"])

                    add_instances = set(new_instance_ids) - set(current_instance_ids)

                    if add_instances:
                        instances_to_add = []
                        for target in params["Targets"]:
                            if target["Id"] in add_instances:
                                tmp_item = {"Id": target["Id"], "Port": target["Port"]}
                                if target.get("AvailabilityZone"):
                                    tmp_item["AvailabilityZone"] = target["AvailabilityZone"]
                                instances_to_add.append(tmp_item)

                        changed = True
                        try:
                            connection.register_targets(
                                TargetGroupArn=target_group["TargetGroupArn"], Targets=instances_to_add, aws_retry=True
                            )
                        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                            module.fail_json_aws(e, msg="Couldn't register targets")

                        if module.params.get("wait"):
                            status_achieved, registered_instances = wait_for_status(
                                connection, module, target_group["TargetGroupArn"], instances_to_add, "healthy"
                            )
                            if not status_achieved:
                                module.fail_json(
                                    msg="Error waiting for target registration to be healthy - please check the AWS console"
                                )

                    remove_instances = set(current_instance_ids) - set(new_instance_ids)

                    if remove_instances:
                        instances_to_remove = []
                        for target in current_targets["TargetHealthDescriptions"]:
                            if target["Target"]["Id"] in remove_instances:
                                instances_to_remove.append(
                                    {"Id": target["Target"]["Id"], "Port": target["Target"]["Port"]}
                                )

                        changed = True
                        try:
                            connection.deregister_targets(
                                TargetGroupArn=target_group["TargetGroupArn"],
                                Targets=instances_to_remove,
                                aws_retry=True,
                            )
                        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                            module.fail_json_aws(e, msg="Couldn't remove targets")

                        if module.params.get("wait"):
                            status_achieved, registered_instances = wait_for_status(
                                connection, module, target_group["TargetGroupArn"], instances_to_remove, "unused"
                            )
                            if not status_achieved:
                                module.fail_json(
                                    msg="Error waiting for target deregistration - please check the AWS console"
                                )

                # register lambda target
                else:
                    try:
                        changed = False
                        target = module.params.get("targets")[0]
                        if len(current_targets["TargetHealthDescriptions"]) == 0:
                            changed = True
                        else:
                            for item in current_targets["TargetHealthDescriptions"]:
                                if target["Id"] != item["Target"]["Id"]:
                                    changed = True
                                    break  # only one target is possible with lambda

                        if changed:
                            if target.get("Id"):
                                connection.register_targets(
                                    TargetGroupArn=target_group["TargetGroupArn"],
                                    Targets=[{"Id": target["Id"]}],
                                    aws_retry=True,
                                )

                    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                        module.fail_json_aws(e, msg="Couldn't register targets")
            else:
                if target_type != "lambda":
                    current_instances = current_targets["TargetHealthDescriptions"]

                    if current_instances:
                        instances_to_remove = []
                        for target in current_targets["TargetHealthDescriptions"]:
                            instances_to_remove.append({"Id": target["Target"]["Id"], "Port": target["Target"]["Port"]})

                        changed = True
                        try:
                            connection.deregister_targets(
                                TargetGroupArn=target_group["TargetGroupArn"],
                                Targets=instances_to_remove,
                                aws_retry=True,
                            )
                        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                            module.fail_json_aws(e, msg="Couldn't remove targets")

                        if module.params.get("wait"):
                            status_achieved, registered_instances = wait_for_status(
                                connection, module, target_group["TargetGroupArn"], instances_to_remove, "unused"
                            )
                            if not status_achieved:
                                module.fail_json(
                                    msg="Error waiting for target deregistration - please check the AWS console"
                                )

                # remove lambda targets
                else:
                    changed = False
                    if current_targets["TargetHealthDescriptions"]:
                        changed = True
                        # only one target is possible with lambda
                        target_to_remove = current_targets["TargetHealthDescriptions"][0]["Target"]["Id"]
                    if changed:
                        connection.deregister_targets(
                            TargetGroupArn=target_group["TargetGroupArn"],
                            Targets=[{"Id": target_to_remove}],
                            aws_retry=True,
                        )
    else:
        try:
            connection.create_target_group(aws_retry=True, **params)
            changed = True
            new_target_group = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't create target group")

        target_group = get_target_group(connection, module, retry_missing=True)

        if module.params.get("targets"):
            if target_type != "lambda":
                params["Targets"] = module.params.get("targets")
                try:
                    connection.register_targets(
                        TargetGroupArn=target_group["TargetGroupArn"], Targets=params["Targets"], aws_retry=True
                    )
                except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                    module.fail_json_aws(e, msg="Couldn't register targets")

                if module.params.get("wait"):
                    status_achieved, registered_instances = wait_for_status(
                        connection, module, target_group["TargetGroupArn"], params["Targets"], "healthy"
                    )
                    if not status_achieved:
                        module.fail_json(
                            msg="Error waiting for target registration to be healthy - please check the AWS console"
                        )

            else:
                try:
                    target = module.params.get("targets")[0]
                    connection.register_targets(
                        TargetGroupArn=target_group["TargetGroupArn"], Targets=[{"Id": target["Id"]}], aws_retry=True
                    )
                    changed = True
                except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                    module.fail_json_aws(e, msg="Couldn't register targets")

    attributes_update = create_or_update_attributes(connection, module, target_group, new_target_group)

    if attributes_update:
        changed = True

    # Tags - only need to play with tags if tags parameter has been set to something
    if tags is not None:
        # Get tags
        current_tags = get_target_group_tags(connection, module, target_group["TargetGroupArn"])

        # Delete necessary tags
        tags_need_modify, tags_to_delete = compare_aws_tags(
            boto3_tag_list_to_ansible_dict(current_tags), tags, purge_tags
        )
        if tags_to_delete:
            try:
                connection.remove_tags(
                    ResourceArns=[target_group["TargetGroupArn"]], TagKeys=tags_to_delete, aws_retry=True
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Couldn't delete tags from target group")
            changed = True

        # Add/update tags
        if tags_need_modify:
            try:
                connection.add_tags(
                    ResourceArns=[target_group["TargetGroupArn"]],
                    Tags=ansible_dict_to_boto3_tag_list(tags_need_modify),
                    aws_retry=True,
                )
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Couldn't add tags to target group")
            changed = True

    # Get the target group again
    target_group = get_target_group(connection, module)

    # Get the target group attributes again
    target_group.update(get_tg_attributes(connection, module, target_group["TargetGroupArn"]))

    # Convert target_group to snake_case
    snaked_tg = camel_dict_to_snake_dict(target_group)

    snaked_tg["tags"] = boto3_tag_list_to_ansible_dict(
        get_target_group_tags(connection, module, target_group["TargetGroupArn"])
    )

    module.exit_json(changed=changed, **snaked_tg)


def delete_target_group(connection, module):
    changed = False
    tg = get_target_group(connection, module)

    if tg:
        try:
            connection.delete_target_group(TargetGroupArn=tg["TargetGroupArn"], aws_retry=True)
            changed = True
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Couldn't delete target group")

    module.exit_json(changed=changed)


def main():
    protocols_list = ["http", "https", "tcp", "tls", "udp", "tcp_udp", "HTTP", "HTTPS", "TCP", "TLS", "UDP", "TCP_UDP"]
    argument_spec = dict(
        deregistration_delay_timeout=dict(type="int"),
        deregistration_connection_termination=dict(type="bool", default=False),
        health_check_protocol=dict(choices=protocols_list),
        health_check_port=dict(),
        health_check_path=dict(),
        health_check_interval=dict(type="int"),
        health_check_timeout=dict(type="int"),
        healthy_threshold_count=dict(type="int"),
        modify_targets=dict(default=True, type="bool"),
        name=dict(required=True),
        port=dict(type="int"),
        protocol=dict(choices=protocols_list),
        protocol_version=dict(type="str", choices=["GRPC", "HTTP1", "HTTP2"]),
        purge_tags=dict(default=True, type="bool"),
        stickiness_enabled=dict(type="bool"),
        stickiness_type=dict(),
        stickiness_lb_cookie_duration=dict(type="int"),
        stickiness_app_cookie_duration=dict(type="int"),
        stickiness_app_cookie_name=dict(),
        load_balancing_algorithm_type=dict(type="str", choices=["round_robin", "least_outstanding_requests"]),
        state=dict(required=True, choices=["present", "absent"]),
        successful_response_codes=dict(),
        tags=dict(type="dict", aliases=["resource_tags"]),
        target_type=dict(choices=["instance", "ip", "lambda", "alb"]),
        targets=dict(type="list", elements="dict"),
        unhealthy_threshold_count=dict(type="int"),
        vpc_id=dict(),
        preserve_client_ip_enabled=dict(type="bool"),
        proxy_protocol_v2_enabled=dict(type="bool"),
        wait_timeout=dict(type="int", default=200),
        wait=dict(type="bool", default=False),
    )
    required_by = dict(
        health_check_path=["health_check_protocol"],
        successful_response_codes=["health_check_protocol"],
    )
    required_if = [
        ["target_type", "instance", ["protocol", "port", "vpc_id"]],
        ["target_type", "ip", ["protocol", "port", "vpc_id"]],
        ["target_type", "alb", ["protocol", "port", "vpc_id"]],
    ]

    module = AnsibleAWSModule(argument_spec=argument_spec, required_by=required_by, required_if=required_if)

    if module.params.get("target_type") is None:
        module.params["target_type"] = "instance"

    connection = module.client("elbv2", retry_decorator=AWSRetry.jittered_backoff(retries=10))

    if module.params.get("state") == "present":
        if module.params.get("protocol") in ["http", "https", "HTTP", "HTTPS"] and module.params.get(
            "deregistration_connection_termination", None
        ):
            module.fail_json(
                msg="A target group with HTTP/S protocol does not support setting deregistration_connection_termination"
            )

        create_or_update_target_group(connection, module)
    else:
        delete_target_group(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices
short_description: Resource module for devices
description:
  - Manage operation update of the resource devices.
  - Update the attributes of a device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  address:
    description: The address of a device.
    type: str
  floorPlanId:
    description: The floor plan to associate to this device. Null disassociates the
      device from the floorplan.
    type: str
  lat:
    description: The latitude of a device.
    type: float
  lng:
    description: The longitude of a device.
    type: float
  moveMapMarker:
    description: Whether or not to set the latitude and longitude of a device based
      on the new address. Only applies when lat and lng are not specified.
    type: bool
  name:
    description: The name of a device.
    type: str
  notes:
    description: The notes for the device. String. Limited to 255 characters.
    type: str
  serial:
    description: Serial path parameter.
    type: str
  switchProfileId:
    description: The ID of a switch template to bind to the device (for available
      switch templates, see the 'Switch Templates' endpoint). Use null to unbind the
      switch device from the current profile. For a device to be bindable to a switch
      template, it must (1) be a switch, and (2) belong to a network that is bound
      to a configuration template.
    type: str
  tags:
    description: The list of tags of a device.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for devices updateDevice
    description: Complete reference of the updateDevice API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-device
notes:
  - SDK Method used are
    devices.Devices.update_device,
  - Paths used are
    put /devices/{serial},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.devices:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    address: 1600 Pennsylvania Ave
    floorPlanId: g_2176982374
    lat: 37.4180951010362
    lng: -122.098531723022
    moveMapMarker: true
    name: My AP
    notes: My AP's note
    serial: string
    switchProfileId: '1234'
    tags:
      - ' recently-added '
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "name": "string",
      "lat": 0,
      "lng": 0,
      "address": "string",
      "notes": "string",
      "tags": [
        "string"
      ],
      "networkId": "string",
      "serial": "string",
      "model": "string",
      "mac": "string",
      "lanIp": "string",
      "firmware": "string",
      "floorPlanId": "string",
      "details": [
        {
          "name": "string",
          "value": "string"
        }
      ],
      "beaconIdParams": {
        "uuid": "string",
        "major": 0,
        "minor": 0
      }
    }
"""



#!/usr/bin/python
from __future__ import absolute_import, division, print_function


__metaclass__ = type

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = """
module: nxos_hsrp
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: (deprecated, removed after 2028-06-01) Manages HSRP configuration on NX-OS switches.
description:
- Manages HSRP configuration on NX-OS switches.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- HSRP feature needs to be enabled first on the system.
- SVIs must exist before using this module.
- Interface must be a L3 port before using this module.
- HSRP cannot be configured on loopback interfaces.
- MD5 authentication is only possible with HSRPv2 while it is ignored if HSRPv1 is
  used instead, while it will not raise any error. Here we allow MD5 authentication
  only with HSRPv2 in order to enforce better practice.
options:
  group:
    description:
    - HSRP group number.
    required: true
    type: str
  interface:
    description:
    - Full name of interface that is being managed for HSRP.
    required: true
    type: str
  version:
    description:
    - HSRP version.
    default: '1'
    choices:
    - '1'
    - '2'
    type: str
  priority:
    description:
    - HSRP priority or keyword 'default'.
    type: str
  preempt:
    description:
    - Enable/Disable preempt.
    choices:
    - enabled
    - disabled
    type: str
  vip:
    description:
    - HSRP virtual IP address or keyword 'default'
    type: str
  auth_string:
    description:
    - Authentication string. If this needs to be hidden(for md5 type), the string
      should be 7 followed by the key string. Otherwise, it can be 0 followed by key
      string or just key string (for backward compatibility). For text type, this
      should be just be a key string. if this is 'default', authentication is removed.
    type: str
  auth_type:
    description:
    - Authentication type.
    choices:
    - text
    - md5
    type: str
  state:
    description:
    - Specify desired state of the resource.
    choices:
    - present
    - absent
    default: present
    type: str
"""

EXAMPLES = """
- name: Ensure HSRP is configured with following params on a SVI
  cisco.nxos.nxos_hsrp:
    group: 10
    vip: 10.1.1.1
    priority: 150
    interface: vlan10
    preempt: enabled

- name: Ensure HSRP is configured with following params on a SVI with clear text authentication
  cisco.nxos.nxos_hsrp:
    group: 10
    vip: 10.1.1.1
    priority: 150
    interface: vlan10
    preempt: enabled
    auth_type: text
    auth_string: CISCO

- name: Ensure HSRP is configured with md5 authentication and clear authentication
    string
  cisco.nxos.nxos_hsrp:
    group: 10
    vip: 10.1.1.1
    priority: 150
    interface: vlan10
    preempt: enabled
    auth_type: md5
    auth_string: 0 1234

- name: Ensure HSRP is configured with md5 authentication and hidden authentication
    string
  cisco.nxos.nxos_hsrp:
    group: 10
    vip: 10.1.1.1
    priority: 150
    interface: vlan10
    preempt: enabled
    auth_type: md5
    auth_string: 7 1234

- name: Remove HSRP config for given interface, group, and VIP
  cisco.nxos.nxos_hsrp:
    group: 10
    interface: vlan10
    vip: 10.1.1.1
    state: absent
"""

RETURN = r"""
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["interface vlan10", "hsrp version 2", "hsrp 30", "ip 10.30.1.1"]
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_capabilities,
    get_interface_type,
    load_config,
    run_commands,
)


PARAM_TO_DEFAULT_KEYMAP = {
    "vip": None,
    "priority": "100",
    "auth_type": "text",
    "auth_string": "cisco",
}


def apply_key_map(key_map, table):
    new_dict = {}
    for key in table:
        new_key = key_map.get(key)
        if new_key:
            value = table.get(key)
            if value:
                new_dict[new_key] = str(value)
            else:
                new_dict[new_key] = value
    return new_dict


def get_interface_mode(interface, intf_type, module):
    command = "show interface {0} | json".format(interface)
    interface = {}
    mode = "unknown"
    try:
        body = run_commands(module, [command])[0]
    except IndexError:
        return None

    if intf_type in ["ethernet", "portchannel"]:
        interface_table = body["TABLE_interface"]["ROW_interface"]
        mode = str(interface_table.get("eth_mode", "layer3"))
        if mode == "access" or mode == "trunk":
            mode = "layer2"
    elif intf_type == "svi":
        mode = "layer3"
    return mode


def get_hsrp_group(group, interface, module):
    command = "show hsrp group {0} all | json".format(group)
    hsrp = {}

    hsrp_key = {
        "sh_if_index": "interface",
        "sh_group_num": "group",
        "sh_group_version": "version",
        "sh_cfg_prio": "priority",
        "sh_preempt": "preempt",
        "sh_vip": "vip",
        "sh_authentication_type": "auth_type",
        "sh_keystring_attr": "auth_enc",
        "sh_authentication_data": "auth_string",
    }

    try:
        body = run_commands(module, [command])[0]
        hsrp_table = body["TABLE_grp_detail"]["ROW_grp_detail"]
        if "sh_keystring_attr" not in hsrp_table:
            del hsrp_key["sh_keystring_attr"]
        if "unknown enum:" in str(hsrp_table):
            hsrp_table = get_hsrp_group_unknown_enum(module, command, hsrp_table)
    except (AttributeError, IndexError, TypeError, KeyError):
        return {}

    if isinstance(hsrp_table, dict):
        hsrp_table = [hsrp_table]

    for hsrp_group in hsrp_table:
        parsed_hsrp = apply_key_map(hsrp_key, hsrp_group)

        parsed_hsrp["interface"] = parsed_hsrp["interface"].lower()

        if parsed_hsrp["version"] == "v1":
            parsed_hsrp["version"] = "1"
        elif parsed_hsrp["version"] == "v2":
            parsed_hsrp["version"] = "2"

        if parsed_hsrp["auth_type"] == "md5":
            if parsed_hsrp["auth_enc"] == "hidden":
                parsed_hsrp["auth_enc"] = "7"
            else:
                parsed_hsrp["auth_enc"] = "0"

        if parsed_hsrp["interface"] == interface:
            return parsed_hsrp

    return hsrp


def get_hsrp_group_unknown_enum(module, command, hsrp_table):
    """Some older NXOS images fail to set the attr values when using structured output and
    instead set the values to <unknown enum>. This fallback method is a workaround that
    uses an unstructured (text) request to query the device a second time.
    'sh_preempt' is currently the only attr affected. Add checks for other attrs as needed.
    """
    if "unknown enum:" in hsrp_table["sh_preempt"]:
        cmd = {"output": "text", "command": command.split("|")[0]}
        out = run_commands(module, cmd)[0]
        hsrp_table["sh_preempt"] = "enabled" if ("may preempt" in out) else "disabled"
    return hsrp_table


def get_commands_remove_hsrp(group, interface):
    commands = ["interface {0}".format(interface), "no hsrp {0}".format(group)]
    return commands


def get_commands_config_hsrp(delta, interface, args, existing):
    commands = []

    config_args = {
        "group": "hsrp {group}",
        "priority": "{priority}",
        "preempt": "{preempt}",
        "vip": "{vip}",
    }

    preempt = delta.get("preempt", None)
    group = delta.get("group", None)
    vip = delta.get("vip", None)
    priority = delta.get("priority", None)

    if preempt:
        if preempt == "enabled":
            delta["preempt"] = "preempt"
        elif preempt == "disabled":
            delta["preempt"] = "no preempt"

    if priority:
        if priority == "default":
            if existing and existing.get("priority") != PARAM_TO_DEFAULT_KEYMAP.get("priority"):
                delta["priority"] = "no priority"
            else:
                del delta["priority"]
        else:
            delta["priority"] = "priority {0}".format(delta["priority"])

    if vip:
        if vip == "default":
            if existing and existing.get("vip") != PARAM_TO_DEFAULT_KEYMAP.get("vip"):
                delta["vip"] = "no ip"
            else:
                del delta["vip"]
        else:
            delta["vip"] = "ip {0}".format(delta["vip"])

    for key in delta:
        command = config_args.get(key, "DNE").format(**delta)
        if command and command != "DNE":
            if key == "group":
                commands.insert(0, command)
            else:
                commands.append(command)
        command = None

    auth_type = delta.get("auth_type", None)
    auth_string = delta.get("auth_string", None)
    auth_enc = delta.get("auth_enc", None)
    if auth_type or auth_string:
        if not auth_type:
            auth_type = args["auth_type"]
        elif not auth_string:
            auth_string = args["auth_string"]
        if auth_string != "default":
            if auth_type == "md5":
                command = "authentication md5 key-string {0} {1}".format(auth_enc, auth_string)
                commands.append(command)
            elif auth_type == "text":
                command = "authentication text {0}".format(auth_string)
                commands.append(command)
        else:
            if existing and existing.get("auth_string") != PARAM_TO_DEFAULT_KEYMAP.get(
                "auth_string",
            ):
                commands.append("no authentication")

    if commands and not group:
        commands.insert(0, "hsrp {0}".format(args["group"]))

    version = delta.get("version", None)
    if version:
        if version == "2":
            command = "hsrp version 2"
        elif version == "1":
            command = "hsrp version 1"
        commands.insert(0, command)
        commands.insert(0, "interface {0}".format(interface))

    if commands:
        if not commands[0].startswith("interface"):
            commands.insert(0, "interface {0}".format(interface))

    return commands


def is_default(interface, module):
    command = "show run interface {0}".format(interface)

    try:
        body = run_commands(module, [command], check_rc=False)[0]
        if "invalid" in body.lower():
            return "DNE"
        else:
            raw_list = body.split("\n")
            if raw_list[-1].startswith("interface"):
                return True
            else:
                return False
    except KeyError:
        return "DNE"


def validate_config(body, vip, module):
    new_body = "".join(body)
    if "invalid ip address" in new_body.lower():
        module.fail_json(msg="Invalid VIP. Possible duplicate IP address.", vip=vip)


def main():
    argument_spec = dict(
        group=dict(required=True, type="str"),
        interface=dict(required=True),
        version=dict(choices=["1", "2"], default="1", required=False),
        priority=dict(type="str", required=False),
        preempt=dict(type="str", choices=["disabled", "enabled"], required=False),
        vip=dict(type="str", required=False),
        auth_type=dict(choices=["text", "md5"], required=False),
        auth_string=dict(type="str", required=False),
        state=dict(choices=["absent", "present"], required=False, default="present"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()
    results = dict(changed=False, warnings=warnings)

    interface = module.params["interface"].lower()
    group = module.params["group"]
    version = module.params["version"]
    state = module.params["state"]
    priority = module.params["priority"]
    preempt = module.params["preempt"]
    vip = module.params["vip"]
    auth_type = module.params["auth_type"]
    auth_full_string = module.params["auth_string"]
    auth_enc = "0"
    auth_string = None
    if auth_full_string:
        kstr = auth_full_string.split()
        if len(kstr) == 2:
            auth_enc = kstr[0]
            auth_string = kstr[1]
        elif len(kstr) == 1:
            auth_string = kstr[0]
        else:
            module.fail_json(msg="Invalid auth_string")
        if auth_enc != "0" and auth_enc != "7":
            module.fail_json(msg="Invalid auth_string, only 0 or 7 allowed")

    device_info = get_capabilities(module)
    network_api = device_info.get("network_api", "nxapi")

    intf_type = get_interface_type(interface)
    if intf_type != "ethernet" and network_api == "cliconf":
        if is_default(interface, module) == "DNE":
            module.fail_json(
                msg="That interface does not exist yet. Create " "it first.",
                interface=interface,
            )
        if intf_type == "loopback":
            module.fail_json(
                msg="Loopback interfaces don't support HSRP.",
                interface=interface,
            )

    mode = get_interface_mode(interface, intf_type, module)
    if mode == "layer2":
        module.fail_json(
            msg="That interface is a layer2 port.\nMake it " "a layer 3 port first.",
            interface=interface,
        )

    if auth_type or auth_string:
        if not (auth_type and auth_string):
            module.fail_json(
                msg="When using auth parameters, you need BOTH " "auth_type AND auth_string.",
            )

    args = dict(
        group=group,
        version=version,
        priority=priority,
        preempt=preempt,
        vip=vip,
        auth_type=auth_type,
        auth_string=auth_string,
        auth_enc=auth_enc,
    )

    proposed = dict((k, v) for k, v in args.items() if v is not None)

    existing = get_hsrp_group(group, interface, module)

    # This will enforce better practice with md5 and hsrp version.
    if proposed.get("auth_type", None) == "md5":
        if proposed["version"] == "1":
            module.fail_json(msg="It's recommended to use HSRP v2 " "when auth_type=md5")

    elif not proposed.get("auth_type", None) and existing:
        if (proposed["version"] == "1" and existing["auth_type"] == "md5") and state == "present":
            module.fail_json(
                msg="Existing auth_type is md5. It's recommended " "to use HSRP v2 when using md5",
            )

    commands = []
    if state == "present":
        delta = dict(set(proposed.items()).difference(existing.items()))
        if delta:
            command = get_commands_config_hsrp(delta, interface, args, existing)
            commands.extend(command)

    elif state == "absent":
        if existing:
            command = get_commands_remove_hsrp(group, interface)
            commands.extend(command)

    if commands:
        if module.check_mode:
            module.exit_json(**results)
        else:
            load_config(module, commands)

            # validate IP
            if network_api == "cliconf" and state == "present":
                commands.insert(0, "config t")
                body = run_commands(module, commands)
                validate_config(body, vip, module)

            results["changed"] = True

            if "configure" in commands:
                commands.pop(0)

    results["commands"] = commands
    module.exit_json(**results)


if __name__ == "__main__":
    main()
